# mypy: disable-error-code="method-assign,no-untyped-def"

import pytest
from rdflib import RDF
from rdflib import RDFS
from rdflib import XSD
from rdflib import Literal
from rdflib import URIRef

from dsp_tools.commands.validate_data.models.rdf_like_data import MigrationMetadata
from dsp_tools.commands.validate_data.models.rdf_like_data import PropertyObject
from dsp_tools.commands.validate_data.models.rdf_like_data import RdfLikeResource
from dsp_tools.commands.validate_data.models.rdf_like_data import RdfLikeValue
from dsp_tools.commands.validate_data.models.rdf_like_data import TripleObjectType
from dsp_tools.commands.validate_data.models.rdf_like_data import TriplePropertyType
from dsp_tools.commands.validate_data.prepare_data.make_data_graph import _make_one_rdflib_object
from dsp_tools.commands.validate_data.prepare_data.make_data_graph import _make_one_resource
from dsp_tools.commands.validate_data.prepare_data.make_data_graph import _make_one_value
from dsp_tools.commands.validate_data.prepare_data.make_data_graph import _make_property_objects_graph
from dsp_tools.utils.rdflib_constants import API_SHAPES
from dsp_tools.utils.rdflib_constants import DATA
from dsp_tools.utils.rdflib_constants import KNORA_API
from dsp_tools.utils.rdflib_constants import KNORA_API_STR
from dsp_tools.utils.rdflib_constants import SubjectObjectTypeAlias
from dsp_tools.utils.xml_parsing.models.parsed_resource import KnoraValueType
from test.unittests.commands.validate_data.constants import ONTO

LABEL_TRIPLE = PropertyObject(TriplePropertyType.RDFS_LABEL, "lbl", TripleObjectType.STRING)
TYPE_TRIPLE = PropertyObject(
    TriplePropertyType.RDF_TYPE, "http://0.0.0.0:3333/ontology/9999/onto/v2#ClassWithEverything", TripleObjectType.IRI
)

UNREIFIED_TRIPLE_OBJECTS = [LABEL_TRIPLE, TYPE_TRIPLE]


RES_IRI = DATA["id"]
RESOURCE_TYPE_STR = "http://0.0.0.0:3333/ontology/9999/onto/v2#ClassWithEverything"


@pytest.fixture
def rdf_like_boolean_value_corr() -> RdfLikeValue:
    return RdfLikeValue(
        "http://0.0.0.0:3333/ontology/9999/onto/v2#testBoolean", "false", KnoraValueType.BOOLEAN_VALUE, []
    )


@pytest.mark.parametrize(
    ("trpl_obj", "object_type", "prop_type", "expected"),
    [
        (
            "label",
            TripleObjectType.STRING,
            None,
            Literal("label", datatype=XSD.string),
        ),
        (
            RESOURCE_TYPE_STR,
            TripleObjectType.IRI,
            None,
            URIRef(RESOURCE_TYPE_STR),
        ),
        (
            None,
            TripleObjectType.IRI,
            None,
            Literal("", datatype=XSD.string),
        ),
        (
            "res_id",
            TripleObjectType.IRI,
            TriplePropertyType.KNORA_STANDOFF_LINK,
            DATA.res_id,
        ),
    ],
)
def test_make_one_rdflib_object(
    trpl_obj: str | None,
    object_type: TripleObjectType,
    prop_type: TriplePropertyType | None,
    expected: SubjectObjectTypeAlias,
):
    result = _make_one_rdflib_object(trpl_obj, object_type, prop_type)
    assert result == expected


class TestResource:
    def test_empty(self):
        res = RdfLikeResource(
            res_id="id",
            property_objects=UNREIFIED_TRIPLE_OBJECTS,
            values=[],
            migration_metadata=MigrationMetadata(),
        )
        res_g = _make_one_resource(res)
        assert len(res_g) == 2
        assert next(res_g.objects(RES_IRI, RDF.type)) == ONTO.ClassWithEverything
        assert next(res_g.objects(RES_IRI, RDFS.label)) == Literal("lbl", datatype=XSD.string)

    def test_with_props(self, rdf_like_boolean_value_corr):
        res = RdfLikeResource(
            res_id="id",
            property_objects=UNREIFIED_TRIPLE_OBJECTS,
            values=[rdf_like_boolean_value_corr],
            migration_metadata=MigrationMetadata(),
        )
        res_g = _make_one_resource(res)
        assert len(res_g) == 5
        assert next(res_g.objects(RES_IRI, RDF.type)) == ONTO.ClassWithEverything
        assert next(res_g.objects(RES_IRI, RDFS.label)) == Literal("lbl", datatype=XSD.string)
        bool_bn = next(res_g.objects(RES_IRI, ONTO.testBoolean))
        assert next(res_g.objects(bool_bn, KNORA_API.booleanValueAsBoolean)) == Literal(False, datatype=XSD.boolean)


class TestBooleanValue:
    def test_corr(self, rdf_like_boolean_value_corr):
        val_g = _make_one_value(rdf_like_boolean_value_corr, RES_IRI)
        assert len(val_g) == 3
        bn = next(val_g.objects(RES_IRI, ONTO.testBoolean))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.BooleanValue
        assert next(val_g.objects(bn, KNORA_API.booleanValueAsBoolean)) == Literal("false", datatype=XSD.boolean)

    def test_one(self):
        val = RdfLikeValue(
            "http://0.0.0.0:3333/ontology/9999/onto/v2#testBoolean", "1", KnoraValueType.BOOLEAN_VALUE, []
        )
        val_g = _make_one_value(val, RES_IRI)
        assert len(val_g) == 3
        bn = next(val_g.objects(RES_IRI, ONTO.testBoolean))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.BooleanValue
        assert next(val_g.objects(bn, KNORA_API.booleanValueAsBoolean)) == Literal("true", datatype=XSD.boolean)

    def test_zero(self):
        val = RdfLikeValue(
            "http://0.0.0.0:3333/ontology/9999/onto/v2#testBoolean", "0", KnoraValueType.BOOLEAN_VALUE, []
        )
        val_g = _make_one_value(val, RES_IRI)
        assert len(val_g) == 3
        bn = next(val_g.objects(RES_IRI, ONTO.testBoolean))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.BooleanValue
        assert next(val_g.objects(bn, KNORA_API.booleanValueAsBoolean)) == Literal("false", datatype=XSD.boolean)


class TestColorValue:
    def test_corr(self):
        val = RdfLikeValue(
            "http://0.0.0.0:3333/ontology/9999/onto/v2#testColor", "#00ff00", KnoraValueType.COLOR_VALUE, []
        )
        val_g = _make_one_value(val, RES_IRI)
        assert len(val_g) == 3
        bn = next(val_g.objects(RES_IRI, ONTO.testColor))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.ColorValue
        assert next(val_g.objects(bn, KNORA_API.colorValueAsColor)) == Literal("#00ff00", datatype=XSD.string)


class TestDateValue:
    def test_corr(self):
        val = RdfLikeValue(
            "http://0.0.0.0:3333/ontology/9999/onto/v2#testSubDate1",
            "JULIAN:BCE:0700:BCE:0600",
            KnoraValueType.DATE_VALUE,
            [],
        )
        val_g = _make_one_value(val, RES_IRI)
        assert len(val_g) == 3
        bn = next(val_g.objects(RES_IRI, ONTO.testSubDate1))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.DateValue
        assert next(val_g.objects(bn, KNORA_API.valueAsString)) == Literal(
            "JULIAN:BCE:0700:BCE:0600", datatype=XSD.string
        )


class TestDecimalValue:
    def test_corr(self):
        val = RdfLikeValue(
            "http://0.0.0.0:3333/ontology/9999/onto/v2#testDecimalSimpleText",
            "1.2",
            KnoraValueType.DECIMAL_VALUE,
            [],
        )
        val_g = _make_one_value(val, RES_IRI)
        assert len(val_g) == 3
        bn = next(val_g.objects(RES_IRI, ONTO.testDecimalSimpleText))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.DecimalValue
        assert next(val_g.objects(bn, KNORA_API.decimalValueAsDecimal)) == Literal("1.2", datatype=XSD.decimal)


class TestGeonameValue:
    def test_corr(self):
        val = RdfLikeValue(
            "http://0.0.0.0:3333/ontology/9999/onto/v2#testGeoname",
            "1241345",
            KnoraValueType.GEONAME_VALUE,
            [],
        )
        val_g = _make_one_value(val, RES_IRI)
        assert len(val_g) == 3
        bn = next(val_g.objects(RES_IRI, ONTO.testGeoname))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.GeonameValue
        assert next(val_g.objects(bn, KNORA_API.geonameValueAsGeonameCode)) == Literal("1241345", datatype=XSD.string)


class TestIntValue:
    def test_corr(self):
        val = RdfLikeValue(
            "http://0.0.0.0:3333/ontology/9999/onto/v2#testIntegerSimpleText",
            "1",
            KnoraValueType.INT_VALUE,
            [],
        )
        val_g = _make_one_value(val, RES_IRI)
        assert len(val_g) == 3
        bn = next(val_g.objects(RES_IRI, ONTO.testIntegerSimpleText))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.IntValue
        assert next(val_g.objects(bn, KNORA_API.intValueAsInt)) == Literal("1", datatype=XSD.integer)


class TestIntervalValue:
    def test_corr(self):
        seg_start = PropertyObject(TriplePropertyType.KNORA_INTERVAL_START, "1", TripleObjectType.DECIMAL)
        seg_end = PropertyObject(TriplePropertyType.KNORA_INTERVAL_END, "2", TripleObjectType.DECIMAL)
        val = RdfLikeValue(
            user_facing_prop=f"{KNORA_API_STR}hasSegmentBounds",
            user_facing_value=None,
            knora_type=KnoraValueType.INTERVAL_VALUE,
            value_metadata=[seg_start, seg_end],
        )
        val_g = _make_one_value(val, RES_IRI)
        assert len(val_g) == 4
        bn = next(val_g.objects(RES_IRI, KNORA_API.hasSegmentBounds))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.IntervalValue
        assert next(val_g.objects(bn, KNORA_API.intervalValueHasStart)) == Literal("1", datatype=XSD.decimal)
        assert next(val_g.objects(bn, KNORA_API.intervalValueHasEnd)) == Literal("2", datatype=XSD.decimal)


class TestLinkValue:
    def test_corr(self):
        val = RdfLikeValue(
            "http://0.0.0.0:3333/ontology/9999/onto/v2#testHasLinkTo",
            "link-id",
            KnoraValueType.LINK_VALUE,
            [],
        )
        val_g = _make_one_value(val, RES_IRI)
        assert len(val_g) == 3
        bn = next(val_g.objects(RES_IRI, ONTO.testHasLinkTo))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.LinkValue
        assert next(val_g.objects(bn, API_SHAPES.linkValueHasTargetID)) == DATA["link-id"]

    def test_none(self):
        val = RdfLikeValue(
            "http://0.0.0.0:3333/ontology/9999/onto/v2#testHasLinkTo", None, KnoraValueType.LINK_VALUE, []
        )
        val_g = _make_one_value(val, RES_IRI)
        assert len(val_g) == 3
        bn = next(val_g.objects(RES_IRI, ONTO.testHasLinkTo))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.LinkValue
        assert next(val_g.objects(bn, API_SHAPES.linkValueHasTargetID)) == DATA[""]


class TestListValue:
    def test_corr(self):
        val = RdfLikeValue(
            "http://0.0.0.0:3333/ontology/9999/onto/v2#testListProp", "n1", KnoraValueType.LIST_VALUE, []
        )
        val_g = _make_one_value(val, RES_IRI)
        assert len(val_g) == 3
        bn = next(val_g.objects(RES_IRI, ONTO.testListProp))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.ListValue
        assert next(val_g.objects(bn, KNORA_API.listValueAsListNode)) == Literal("n1", datatype=XSD.string)


class TestSimpleTextValue:
    def test_corr(self):
        val = RdfLikeValue(
            "http://0.0.0.0:3333/ontology/9999/onto/v2#testTextarea",
            "simple text",
            KnoraValueType.SIMPLETEXT_VALUE,
            [],
        )
        val_g = _make_one_value(val, RES_IRI)
        assert len(val_g) == 3
        bn = next(val_g.objects(RES_IRI, ONTO.testTextarea))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.TextValue
        assert next(val_g.objects(bn, KNORA_API.valueAsString)) == Literal("simple text", datatype=XSD.string)


class TestRichtextValue:
    def test_corr(self):
        val = RdfLikeValue(
            "http://0.0.0.0:3333/ontology/9999/onto/v2#testRichtext",
            "rich text",
            KnoraValueType.RICHTEXT_VALUE,
            [],
        )
        val_g = _make_one_value(val, RES_IRI)
        assert len(val_g) == 3
        bn = next(val_g.objects(RES_IRI, ONTO.testRichtext))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.TextValue
        assert next(val_g.objects(bn, KNORA_API.textValueAsXml)) == Literal("rich text", datatype=XSD.string)


class TestTimeValue:
    def test_corr(self):
        val = RdfLikeValue(
            "http://0.0.0.0:3333/ontology/9999/onto/v2#testTimeValue",
            "2019-10-23T13:45:12.01-14:00",
            KnoraValueType.TIME_VALUE,
            [],
        )
        val_g = _make_one_value(val, RES_IRI)
        assert len(val_g) == 3
        bn = next(val_g.objects(RES_IRI, ONTO.testTimeValue))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.TimeValue
        assert next(val_g.objects(bn, KNORA_API.timeValueAsTimeStamp)) == Literal(
            "2019-10-23T13:45:12.01-14:00", datatype=XSD.dateTimeStamp
        )


class TestUriValue:
    def test_corr(self):
        val = RdfLikeValue(
            "http://0.0.0.0:3333/ontology/9999/onto/v2#testUriValue",
            "https://dasch.swiss",
            KnoraValueType.URI_VALUE,
            [],
        )
        val_g = _make_one_value(val, RES_IRI)
        assert len(val_g) == 3
        bn = next(val_g.objects(RES_IRI, ONTO.testUriValue))
        assert next(val_g.objects(bn, RDF.type)) == KNORA_API.UriValue
        assert next(val_g.objects(bn, KNORA_API.uriValueAsUri)) == Literal("https://dasch.swiss", datatype=XSD.anyURI)


def test_make_property_objects_graph():
    test_prop_obj = PropertyObject(TriplePropertyType.RDF_TYPE, RESOURCE_TYPE_STR, TripleObjectType.IRI)
    graph = _make_property_objects_graph([test_prop_obj], RES_IRI)
    assert len(graph) == 1
    assert next(graph.objects(RES_IRI, RDF.type)) == URIRef(RESOURCE_TYPE_STR)


class TestFileValue:
    def test_make_file_value_graph_real_file(self):
        file_value = RdfLikeValue(
            user_facing_prop=f"{KNORA_API_STR}hasArchiveFileValue",
            user_facing_value="test.zip",
            knora_type=KnoraValueType.ARCHIVE_FILE,
            value_metadata=[],
        )
        file_g = _make_one_value(file_value, RES_IRI)
        assert len(file_g) == 3
        bn = next(file_g.objects(RES_IRI, KNORA_API.hasArchiveFileValue))
        assert next(file_g.objects(bn, RDF.type)) == KNORA_API.ArchiveFileValue
        assert next(file_g.objects(bn, KNORA_API.fileValueHasFilename)) == Literal("test.zip", datatype=XSD.string)

    def test_make_file_value_graph_iiif_uri(self):
        uri = "https://iiif.wellcomecollection.org/1Oi7mdiLsG7-FmFgp0xz2xU.jp2/full/max/0/default.jpg"
        file_value = RdfLikeValue(
            user_facing_prop=f"{KNORA_API_STR}hasStillImageFileValue",
            user_facing_value=uri,
            knora_type=KnoraValueType.STILL_IMAGE_IIIF,
            value_metadata=[],
        )
        file_g = _make_one_value(file_value, RES_IRI)
        assert len(file_g) == 3
        bn = next(file_g.objects(RES_IRI, KNORA_API.hasStillImageFileValue))
        assert next(file_g.objects(bn, RDF.type)) == KNORA_API.StillImageExternalFileValue
        assert next(file_g.objects(bn, KNORA_API.stillImageFileValueHasExternalUrl)) == Literal(
            uri, datatype=XSD.anyURI
        )


if __name__ == "__main__":
    pytest.main([__file__])



from __future__ import annotations

from pathlib import Path

from loguru import logger
from lxml import etree

from dsp_tools.cli.args import ServerCredentials
from dsp_tools.cli.args import ValidateDataConfig
from dsp_tools.cli.args import ValidationSeverity
from dsp_tools.clients.authentication_client import AuthenticationClient
from dsp_tools.clients.authentication_client_live import AuthenticationClientLive
from dsp_tools.clients.connection import Connection
from dsp_tools.clients.connection_live import ConnectionLive
from dsp_tools.clients.legal_info_client_live import LegalInfoClientLive
from dsp_tools.commands.ingest_xmlupload.create_resources.apply_ingest_id import get_mapping_dict_from_file
from dsp_tools.commands.ingest_xmlupload.create_resources.apply_ingest_id import replace_filepath_with_internal_filename
from dsp_tools.commands.validate_data.validate_data import validate_parsed_resources
from dsp_tools.commands.xmlupload.models.ingest import BulkIngestedAssetClient
from dsp_tools.commands.xmlupload.models.upload_clients import UploadClients
from dsp_tools.commands.xmlupload.models.upload_state import UploadState
from dsp_tools.commands.xmlupload.prepare_xml_input.get_processed_resources import get_processed_resources
from dsp_tools.commands.xmlupload.prepare_xml_input.list_client import ListClientLive
from dsp_tools.commands.xmlupload.prepare_xml_input.prepare_xml_input import get_parsed_resources_and_mappers
from dsp_tools.commands.xmlupload.prepare_xml_input.prepare_xml_input import get_stash_and_upload_order
from dsp_tools.commands.xmlupload.prepare_xml_input.read_validate_xml_file import validate_iiif_uris
from dsp_tools.commands.xmlupload.project_client import ProjectClientLive
from dsp_tools.commands.xmlupload.upload_config import UploadConfig
from dsp_tools.commands.xmlupload.xmlupload import enable_unknown_license_if_any_are_missing
from dsp_tools.commands.xmlupload.xmlupload import execute_upload
from dsp_tools.error.exceptions import InputError
from dsp_tools.utils.ansi_colors import BOLD_RED
from dsp_tools.utils.ansi_colors import RESET_TO_DEFAULT
from dsp_tools.utils.data_formats.uri_util import is_prod_like_server
from dsp_tools.utils.xml_parsing.parse_clean_validate_xml import parse_and_clean_xml_file


def ingest_xmlupload(
    xml_file: Path,
    creds: ServerCredentials,
    interrupt_after: int | None = None,
    skip_validation: bool = False,
) -> bool:
    """
    This function reads an XML file
    and imports the data described in it onto the DSP server,
    using the ingest XML upload method.
    Before using this function,
    the multimedia files must be ingested on the DSP server.
    A mapping file with the internal IDs of the multimedia files must also be provided.

    Args:
        xml_file: path to XML file containing the resources
        creds: credentials to access the DSP server
        interrupt_after: if set, the upload will be interrupted after this number of resources
        skip_validation: skip the SHACL validation

    Returns:
        True if all resources could be uploaded without errors; False if one of the resources could not be
        uploaded because there is an error in it

    Raises:
        InputError: if any media was not uploaded or uploaded media was not referenced.
    """
    root = parse_and_clean_xml_file(xml_file)
    shortcode = root.attrib["shortcode"]
    root = _replace_filepaths_with_internal_filename_from_ingest(root, shortcode)

    auth = AuthenticationClientLive(server=creds.server, email=creds.user, password=creds.password)
    con = ConnectionLive(creds.server, auth)
    config = UploadConfig(
        media_previously_uploaded=True,
        interrupt_after=interrupt_after,
    ).with_server_info(
        server=creds.server,
        shortcode=shortcode,
    )
    clients = _get_live_clients(con, config, auth)

    parsed_resources, lookups = get_parsed_resources_and_mappers(root, clients)
    validation_should_be_skipped = skip_validation
    is_on_prod_like_server = is_prod_like_server(creds.server)
    if is_on_prod_like_server and config.skip_validation:
        msg = (
            "You set the flag '--skip-validation' to circumvent the SHACL schema validation. "
            "This means that the upload may fail due to undetected errors. "
            "Do you wish to skip the validation (yes/no)? "
        )
        resp = ""
        while resp not in ["yes", "no"]:
            resp = input(BOLD_RED + msg + RESET_TO_DEFAULT)
        if str(resp) == "no":
            validation_should_be_skipped = False
    if not validation_should_be_skipped:
        v_severity = config.validation_severity
        if is_on_prod_like_server:
            v_severity = ValidationSeverity.INFO
        validation_passed = validate_parsed_resources(
            parsed_resources=parsed_resources,
            authorship_lookup=lookups.authorships,
            permission_ids=list(lookups.permissions.keys()),
            shortcode=shortcode,
            config=ValidateDataConfig(
                xml_file,
                save_graph_dir=None,
                severity=v_severity,
                ignore_duplicate_files_warning=True,
                is_on_prod_server=is_on_prod_like_server,
            ),
            auth=auth,
        )
        if not validation_passed:
            return False
    else:
        logger.debug("SHACL validation was skipped.")

    if not config.skip_iiif_validation:
        validate_iiif_uris(root)

    if not is_on_prod_like_server:
        enable_unknown_license_if_any_are_missing(clients.legal_info_client, parsed_resources)

    processed_resources = get_processed_resources(parsed_resources, lookups, is_on_prod_like_server)

    sorted_resources, stash = get_stash_and_upload_order(processed_resources)

    state = UploadState(
        pending_resources=sorted_resources,
        pending_stash=stash,
        config=config,
    )

    return execute_upload(clients, state)


def _replace_filepaths_with_internal_filename_from_ingest(root: etree._Element, shortcode: str) -> etree._Element:
    orig_path_2_asset_id = get_mapping_dict_from_file(shortcode)
    root, ingest_info = replace_filepath_with_internal_filename(root, orig_path_2_asset_id)
    if ok := ingest_info.ok_msg():
        print(ok)
        logger.info(ok)
    else:
        err_msg = ingest_info.execute_error_protocol()
        raise InputError(err_msg)
    return root


def _get_live_clients(con: Connection, config: UploadConfig, auth: AuthenticationClient) -> UploadClients:
    ingest_client = BulkIngestedAssetClient()
    project_client = ProjectClientLive(con, config.shortcode)
    list_client = ListClientLive(con, project_client.get_project_iri())
    legal_info_client = LegalInfoClientLive(config.server, config.shortcode, auth)
    return UploadClients(ingest_client, project_client, list_client, legal_info_client)



from typing import Union

from rdflib import IdentifiedNode
from rdflib import Literal
from rdflib import Namespace
from rdflib import Variable
from rdflib.term import Node

# rdflib typing
type PropertyTypeAlias = Union[IdentifiedNode, Variable]
type SubjectObjectTypeAlias = Union[IdentifiedNode, Literal, Variable, Node]

# Namespaces as string
KNORA_API_STR = "http://api.knora.org/ontology/knora-api/v2#"
API_SHAPES_STR = "http://api.knora.org/ontology/knora-api/shapes/v2#"

# Namespaces
KNORA_API = Namespace(KNORA_API_STR)
API_SHAPES = Namespace(API_SHAPES_STR)
DATA = Namespace("http://data/")

DASH = Namespace("http://datashapes.org/dash#")



from __future__ import annotations

import base64
import uuid

import regex

from dsp_tools.error.exceptions import BaseError


def convert_ark_v0_to_resource_iri(ark: str) -> str:
    """
    Converts an ARK URL from salsah.org (ARK version 0) of the form ark:/72163/080c-779b9990a0c3f-6e
    to a DSP resource IRI of the form http://rdfh.ch/080C/Ef9heHjPWDS7dMR_gGax2Q

    This method is needed for the migration of projects from salsah.org to DSP.
    Resources need to be created with an existing ARK,
    so the IRI needs to be extracted from that ARK
    in order for the ARK URL to be still valid after the migration.

    Args:
        ark: an ARK version 0 of the form ark:/72163/080c-779b9990a0c3f-6e,
            '72163' being the Name Assigning Authority number,
            '080c' being the project shortcode,
            '779b9990a0c3f' being an ID derived from the object's Salsah ID and
            '6e' being check digits

    Raises:
        BaseError: if the ARK is invalid

    Returns:
        Resource IRI (str) of the form http://rdfh.ch/080C/Ef9heHjPWDS7dMR_gGax2Q
    """
    # create the DaSCH namespace to create version 5 UUIDs
    generic_namespace_url = uuid.NAMESPACE_URL
    dasch_uuid_ns = uuid.uuid5(generic_namespace_url, "https://dasch.swiss")  # cace8b00-717e-50d5-bcb9-486f39d733a2

    # get the salsah resource ID from the ARK and convert it to a UUID version 5 (base64 encoded)
    if ark.count("-") != 2:
        raise BaseError(f"Error while converting ARK '{ark}'. The ARK seems to be invalid")
    project_id, resource_id, _ = ark.split("-")
    _, project_id = project_id.rsplit("/", 1)
    project_id = project_id.upper()
    if not regex.match("^[0-9a-fA-F]{4}$", project_id):
        raise BaseError(f"Error while converting ARK '{ark}'. Invalid project shortcode '{project_id}'")
    if not regex.match("^[0-9A-Za-z]+$", resource_id):
        raise BaseError(f"Error while converting ARK '{ark}'. Invalid Salsah ID '{resource_id}'")

    # make a UUID v5 from the namespace created above (which is a UUID itself) and the resource ID
    # and encode it to base64
    dsp_uuid = base64.urlsafe_b64encode(uuid.uuid5(dasch_uuid_ns, resource_id).bytes).decode("utf-8")
    dsp_uuid = dsp_uuid[:-2]

    # use the new UUID to create the resource IRI
    return f"http://rdfh.ch/{project_id}/{dsp_uuid}"



from dataclasses import dataclass
from dataclasses import field


@dataclass
class IriResolver:
    """Service for resolving internal IDs to IRIs."""

    lookup: dict[str, str] = field(default_factory=dict)

    def update(self, internal_id: str, iri: str) -> None:
        """Adds or updates an internal ID to IRI mapping"""
        self.lookup[internal_id] = iri

    def get(self, internal_id: str) -> str | None:
        """Resolves an internal ID to an IRI."""
        return self.lookup.get(internal_id)

    def non_empty(self) -> bool:
        """Checks if the resolver is empty."""
        return bool(self.lookup)



from typing import Union

from rdflib import RDF
from rdflib import XSD
from rdflib import BNode
from rdflib import Graph
from rdflib import Literal
from rdflib import URIRef

from dsp_tools.commands.xmlupload.iri_resolver import IriResolver
from dsp_tools.commands.xmlupload.make_rdf_graph.constants import LINK_PROP_TYPE_INFO
from dsp_tools.commands.xmlupload.make_rdf_graph.constants import LIST_PROP_TYPE_INFO
from dsp_tools.commands.xmlupload.make_rdf_graph.constants import RDF_LITERAL_PROP_TYPE_MAPPER
from dsp_tools.commands.xmlupload.make_rdf_graph.constants import RICHTEXT_PROP_TYPE_INFO
from dsp_tools.commands.xmlupload.models.lookup_models import IRILookups
from dsp_tools.commands.xmlupload.models.permission import Permissions
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedBoolean
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedColor
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedDate
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedDecimal
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedGeometry
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedGeoname
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedInt
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedInterval
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedLink
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedList
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedRichtext
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedSimpleText
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedTime
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedUri
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedValue
from dsp_tools.commands.xmlupload.models.rdf_models import RDFPropTypeInfo
from dsp_tools.config.logger_config import WARNINGS_SAVEPATH
from dsp_tools.error.exceptions import BaseError
from dsp_tools.error.exceptions import InputError
from dsp_tools.utils.data_formats.date_util import DayMonthYearEra
from dsp_tools.utils.data_formats.date_util import SingleDate
from dsp_tools.utils.data_formats.date_util import StartEnd
from dsp_tools.utils.data_formats.iri_util import is_resource_iri
from dsp_tools.utils.rdflib_constants import KNORA_API

type LiteralValueTypesAlias = Union[
    ProcessedBoolean,
    ProcessedColor,
    ProcessedDecimal,
    ProcessedGeoname,
    ProcessedGeometry,
    ProcessedInt,
    ProcessedSimpleText,
    ProcessedTime,
    ProcessedUri,
]


def make_values(values: list[ProcessedValue], res_node: BNode | URIRef, lookups: IRILookups) -> Graph:
    """
    Serialise the values of a resource.

    Args:
        values: list of ProcessedValues of the resource
        res_node: node of the resource
        lookups: lookups to resolve IRIs

    Returns:
        Graph with the values
    """
    properties_graph = Graph()
    for val in values:
        single_prop_graph = _make_one_value_graph(val=val, res_node=res_node, iri_lookups=lookups)
        properties_graph += single_prop_graph
    return properties_graph


def _make_one_value_graph(val: ProcessedValue, res_node: BNode | URIRef, iri_lookups: IRILookups) -> Graph:
    match val:
        case (
            ProcessedBoolean()
            | ProcessedColor()
            | ProcessedDecimal()
            | ProcessedGeometry()
            | ProcessedGeoname()
            | ProcessedInt()
            | ProcessedTime()
            | ProcessedUri()
            | ProcessedSimpleText()
        ):
            literal_info = RDF_LITERAL_PROP_TYPE_MAPPER[type(val)]
            properties_graph = _make_value_graph_with_literal_object(
                val=val,
                res_node=res_node,
                prop_type_info=literal_info,
            )
        case ProcessedList():
            properties_graph = _make_list_value_graph(val=val, res_node=res_node, prop_type_info=LIST_PROP_TYPE_INFO)
        case ProcessedLink():
            target_iri = _resolve_id_to_iri(val.value, iri_lookups.id_to_iri)
            properties_graph = make_link_value_graph(
                val=val,
                val_node=BNode(),
                res_node=res_node,
                target_iri=URIRef(target_iri),
            )
        case ProcessedRichtext():
            properties_graph = make_richtext_value_graph(
                val=val,
                val_node=BNode(),
                res_node=res_node,
                iri_resolver=iri_lookups.id_to_iri,
            )
        case ProcessedDate():
            properties_graph = _make_date_value_graph(
                val=val,
                res_node=res_node,
            )
        case ProcessedInterval():
            properties_graph = _make_interval_value_graph(
                val=val,
                res_node=res_node,
            )
        case _:
            raise InputError(f"Unknown value type: {type(val).__name__}")
    return properties_graph


def _make_base_value_graph(
    val: ProcessedValue,
    val_node: BNode | URIRef,
    prop_type_info: RDFPropTypeInfo,
    res_node: BNode | URIRef,
) -> Graph:
    g = _add_optional_triples(val_node, val.permissions, val.comment)
    g.add((res_node, URIRef(val.prop_iri), val_node))
    g.add((val_node, RDF.type, prop_type_info.knora_type))
    return g


def _add_optional_triples(val_bn: BNode | URIRef, permissions: Permissions | None, comment: str | None) -> Graph:
    g = Graph()
    if permissions:
        g.add((val_bn, KNORA_API.hasPermissions, Literal(str(permissions), datatype=XSD.string)))
    if comment:
        g.add((val_bn, KNORA_API.valueHasComment, Literal(comment, datatype=XSD.string)))
    return g


def _make_value_graph_with_literal_object(
    val: LiteralValueTypesAlias,
    prop_type_info: RDFPropTypeInfo,
    res_node: BNode | URIRef,
) -> Graph:
    val_bn = BNode()
    g = _make_base_value_graph(val=val, val_node=val_bn, prop_type_info=prop_type_info, res_node=res_node)
    g.add((val_bn, prop_type_info.knora_prop, Literal(val.value, datatype=prop_type_info.xsd_type)))
    return g


def _make_list_value_graph(
    val: ProcessedList,
    res_node: BNode | URIRef,
    prop_type_info: RDFPropTypeInfo,
) -> Graph:
    val_bn = BNode()
    g = _make_base_value_graph(val=val, val_node=val_bn, prop_type_info=prop_type_info, res_node=res_node)
    g.add((val_bn, prop_type_info.knora_prop, URIRef(val.value)))
    return g


def make_link_value_graph(
    val: ProcessedLink,
    val_node: BNode | URIRef,
    res_node: BNode | URIRef,
    target_iri: URIRef,
) -> Graph:
    """Make a LinkValue Graph"""
    g = _make_base_value_graph(val=val, val_node=val_node, prop_type_info=LINK_PROP_TYPE_INFO, res_node=res_node)
    g.add((val_node, LINK_PROP_TYPE_INFO.knora_prop, target_iri))
    return g


def _resolve_id_to_iri(value: str, iri_resolver: IriResolver) -> URIRef:
    if is_resource_iri(value):
        return URIRef(value)
    elif resolved_iri := iri_resolver.get(value):
        return URIRef(resolved_iri)
    msg = (
        f"Could not find the ID {value} in the id2iri mapping. "
        f"This is probably because the resource '{value}' could not be created. "
        f"See {WARNINGS_SAVEPATH} for more information."
    )
    raise BaseError(msg)


def _make_date_value_graph(
    val: ProcessedDate,
    res_node: BNode | URIRef,
) -> Graph:
    val_bn = BNode()
    date = val.value
    g = _add_optional_triples(val_bn, val.permissions, val.comment)
    g.add((res_node, URIRef(val.prop_iri), val_bn))
    g.add((val_bn, RDF.type, KNORA_API.DateValue))
    if cal := date.calendar:
        g.add((val_bn, KNORA_API.dateValueHasCalendar, Literal(cal.value, datatype=XSD.string)))
    g += _make_single_date_graph(val_bn, date.start, StartEnd.START)
    if date.end:
        g += _make_single_date_graph(val_bn, date.end, StartEnd.END)
    return g


def _make_single_date_graph(val_bn: BNode, date: SingleDate, start_end: StartEnd) -> Graph:
    def get_prop(precision: DayMonthYearEra) -> URIRef:
        return KNORA_API[f"dateValueHas{start_end.value}{precision.value}"]

    g = Graph()
    if yr := date.year:
        g.add((val_bn, get_prop(DayMonthYearEra.YEAR), Literal(yr, datatype=XSD.integer)))
    if mnt := date.month:
        g.add((val_bn, get_prop(DayMonthYearEra.MONTH), Literal(mnt, datatype=XSD.integer)))
    if day := date.day:
        g.add((val_bn, get_prop(DayMonthYearEra.DAY), Literal(day, datatype=XSD.integer)))
    if era := date.era:
        g.add((val_bn, get_prop(DayMonthYearEra.ERA), Literal(era.value, datatype=XSD.string)))
    return g


def _make_interval_value_graph(
    val: ProcessedInterval,
    res_node: BNode | URIRef,
) -> Graph:
    val_bn = BNode()
    g = _add_optional_triples(val_bn, val.permissions, val.comment)
    g.add((res_node, URIRef(val.prop_iri), val_bn))
    g.add((val_bn, RDF.type, KNORA_API.IntervalValue))
    g.add((val_bn, KNORA_API.intervalValueHasStart, Literal(val.value.start, datatype=XSD.decimal)))
    g.add((val_bn, KNORA_API.intervalValueHasEnd, Literal(val.value.end, datatype=XSD.decimal)))
    return g


def make_richtext_value_graph(
    val: ProcessedRichtext,
    val_node: BNode | URIRef,
    res_node: BNode | URIRef,
    iri_resolver: IriResolver,
) -> Graph:
    """
    Creates an rdflib graph for a richtext value.

    Args:
        val: Richtext value
        val_node: IRI or blank node of the value
        res_node: IRI or blank node of the resource
        iri_resolver: id to IRI resolver

    Returns:
        Graph
    """
    g = _make_base_value_graph(val=val, val_node=val_node, prop_type_info=RICHTEXT_PROP_TYPE_INFO, res_node=res_node)
    xml_with_iris = val.value.with_iris(iri_resolver)
    val_str = xml_with_iris.as_xml()
    g.add((val_node, RICHTEXT_PROP_TYPE_INFO.knora_prop, Literal(val_str, datatype=XSD.string)))
    g.add((val_node, KNORA_API.textValueHasMapping, URIRef("http://rdfh.ch/standoff/mappings/StandardMapping")))
    return g



import json
import warnings
from typing import Any

import regex

from dsp_tools.cli.args import ServerCredentials
from dsp_tools.clients.authentication_client_live import AuthenticationClientLive
from dsp_tools.clients.connection import Connection
from dsp_tools.clients.connection_live import ConnectionLive
from dsp_tools.commands.project.legacy_models.group import Group
from dsp_tools.commands.project.legacy_models.listnode import ListNode
from dsp_tools.commands.project.legacy_models.ontology import Ontology
from dsp_tools.commands.project.legacy_models.project import Project
from dsp_tools.commands.project.legacy_models.user import User
from dsp_tools.commands.project.models.permissions_client import PermissionsClient
from dsp_tools.error.exceptions import BaseError


def get_project(
    project_identifier: str,
    outfile_path: str,
    creds: ServerCredentials,
    verbose: bool = False,
) -> bool:
    """
    This function writes a project from a DSP server into a JSON file.

    Args:
        project_identifier: the project identifier, either shortcode, shortname or IRI of the project
        outfile_path: the output file the JSON content should be written to
        creds: the credentials to access the DSP server
        verbose: verbose option for the command, if used more output is given to the user

    Raises:
        BaseError: if something went wrong

    Returns:
        True if the process finishes without errors
    """
    auth = AuthenticationClientLive(creds.server, creds.user, creds.password)
    try:
        auth.get_token()
        con = ConnectionLive(creds.server, auth)
    except BaseError:
        warnings.warn("WARNING: Missing or wrong credentials. You won't get data about the users of this project.")
        con = ConnectionLive(creds.server)

    project = _create_project(con, project_identifier)

    project = project.read()
    project_obj = project.createDefinitionFileObj()

    project_obj["default_permissions"] = _get_default_permissions(auth, str(project.iri))

    project_obj["groups"] = _get_groups(con, str(project.iri), verbose)

    project_obj["users"] = _get_users(con, project, verbose)

    project_obj["lists"] = _get_lists(con, project, verbose)

    prefixes, ontos = _get_ontologies(con, str(project.iri), verbose)
    project_obj["ontologies"] = ontos

    schema = "https://raw.githubusercontent.com/dasch-swiss/dsp-tools/main/src/dsp_tools/resources/schema/project.json"
    outfile_content = {
        "prefixes": prefixes,
        "$schema": schema,
        "project": project_obj,
    }

    with open(outfile_path, "w", encoding="utf-8") as f:
        json.dump(outfile_content, f, indent=4, ensure_ascii=False)

    return True


def _create_project(con: Connection, project_identifier: str) -> Project:
    if regex.match("[0-9A-F]{4}", project_identifier):  # shortcode
        return Project(con=con, shortcode=project_identifier)
    elif regex.match("^[\\w-]+$", project_identifier):  # shortname
        return Project(con=con, shortname=project_identifier.lower())
    elif regex.match("^(http)s?://([\\w\\.\\-~]+:?\\d{,4})(/[\\w\\-~]+)+$", project_identifier):  # iri
        return Project(con=con, shortname=project_identifier)
    else:
        raise BaseError(
            f"ERROR Invalid project identifier '{project_identifier}'. Use the project's shortcode, shortname or IRI."
        )


def _get_default_permissions(auth: AuthenticationClientLive, project_iri: str) -> str:
    perm_client = PermissionsClient(auth, project_iri)
    project_doaps = perm_client.get_project_doaps()
    result = _parse_default_permissions(project_doaps)
    if result == "unknown":
        return (
            "We cannot determine if this project is public or private. "
            "The DSP-TOOLS devs can assist you in analysing the existing DOAPs, "
            "and help you deciding if the original intent was rather public or rather private."
        )
    return result


def _parse_default_permissions(project_doaps: list[dict[str, Any]]) -> str:  # noqa: PLR0911 (too many return statements)
    """If the DOAPs exactly match our definition of public/private, return public/private. Otherwise, return unknown."""
    unsupported_groups = ("SystemAdmin", "ProjectAdmin", "Creator", "KnownUser", "UnknownUser")
    if [x for x in project_doaps if x.get("forGroup", "").endswith(unsupported_groups)]:
        return "unknown"
    proj_member_doaps = [x for x in project_doaps if x["forGroup"].endswith("ProjectMember")]
    if len(proj_member_doaps) != 1:
        return "unknown"
    perms = proj_member_doaps[0]["hasPermissions"]
    if len(perms) not in [2, 4]:
        return "unknown"
    proj_adm_perms = [x for x in perms if x["additionalInformation"].endswith("ProjectAdmin")]
    proj_mem_perms = [x for x in perms if x["additionalInformation"].endswith("ProjectMember")]
    knwn_usr_perms = [x for x in perms if x["additionalInformation"].endswith("KnownUser")]
    unkn_usr_perms = [x for x in perms if x["additionalInformation"].endswith("UnknownUser")]
    if not (len(proj_adm_perms) == len(proj_mem_perms) == 1):
        return "unknown"
    if proj_adm_perms[0]["name"] != "CR" or proj_mem_perms[0]["name"] != "D":
        return "unknown"
    if len(knwn_usr_perms) == len(unkn_usr_perms) == 0:
        return "private"
    if not (len(knwn_usr_perms) == len(unkn_usr_perms) == 1):
        return "unknown"
    if knwn_usr_perms[0]["name"] != "V" or unkn_usr_perms[0]["name"] != "V":
        return "unknown"
    return "public"


def _get_groups(con: Connection, project_iri: str, verbose: bool) -> list[dict[str, Any]]:
    if verbose:
        print("Getting groups...")
    groups_obj: list[dict[str, Any]] = []
    if groups := Group.getAllGroupsForProject(con=con, proj_iri=project_iri):
        for group in groups:
            groups_obj.append(group.createDefinitionFileObj())
            if verbose:
                print(f"    Got group '{group.name}'")
    return groups_obj


def _get_users(con: Connection, project: Project, verbose: bool) -> list[dict[str, Any]] | None:
    if verbose:
        print("Getting users...")
    try:
        users = User.getAllUsersForProject(con=con, proj_shortcode=str(project.shortcode))
    except BaseError:
        return None
    if users is None:
        return None

    users_obj: list[dict[str, Any]] = []
    for usr in users:
        users_obj.append(
            usr.createDefinitionFileObj(
                con=con,
                proj_shortname=str(project.shortname),
                proj_iri=str(project.iri),
            )
        )
        if verbose:
            print(f"    Got user '{usr.username}'")
    return users_obj


def _get_lists(con: Connection, project: Project, verbose: bool) -> list[dict[str, Any]]:
    if verbose:
        print("Getting lists...")
    list_obj: list[dict[str, Any]] = []
    if list_roots := ListNode.getAllLists(con=con, project_iri=project.iri):
        for list_root in list_roots:
            complete_list = list_root.getAllNodes()
            list_obj.append(complete_list.createDefinitionFileObj())
            if verbose:
                print(f"    Got list '{list_root.name}'")
    return list_obj


def _get_ontologies(con: Connection, project_iri: str, verbose: bool) -> tuple[dict[str, str], list[dict[str, Any]]]:
    if verbose:
        print("Getting ontologies...")
    ontos = []
    prefixes: dict[str, str] = {}
    ontologies = Ontology.getProjectOntologies(con, project_iri)
    ontology_ids = [onto.iri for onto in ontologies]
    for ontology_id in ontology_ids:
        onto_url_parts = ontology_id.split("/")  # an id has the form http://0.0.0.0:3333/ontology/4123/testonto/v2
        name = onto_url_parts[len(onto_url_parts) - 2]
        shortcode = onto_url_parts[len(onto_url_parts) - 3]
        ontology = Ontology.getOntologyFromServer(con=con, shortcode=shortcode, name=name)
        ontos.append(ontology.createDefinitionFileObj())
        prefixes.update(ontology.context.get_externals_used())
        if verbose:
            print(f"    Got ontology '{name}'")
    return prefixes, ontos



import warnings

import pandas as pd
import pytest
import regex

from dsp_tools.error.exceptions import InputError
from dsp_tools.error.xmllib_warnings import XmllibInputInfo
from dsp_tools.error.xmllib_warnings import XmllibInputWarning
from dsp_tools.xmllib.models.config_options import Permissions
from dsp_tools.xmllib.models.internal.values import BooleanValue
from dsp_tools.xmllib.models.internal.values import ColorValue
from dsp_tools.xmllib.models.internal.values import DateValue
from dsp_tools.xmllib.models.internal.values import DecimalValue
from dsp_tools.xmllib.models.internal.values import GeonameValue
from dsp_tools.xmllib.models.internal.values import IntValue
from dsp_tools.xmllib.models.internal.values import LinkValue
from dsp_tools.xmllib.models.internal.values import ListValue
from dsp_tools.xmllib.models.internal.values import Richtext
from dsp_tools.xmllib.models.internal.values import SimpleText
from dsp_tools.xmllib.models.internal.values import TimeValue
from dsp_tools.xmllib.models.internal.values import UriValue


class TestBooleanValue:
    def test_good(self) -> None:
        with warnings.catch_warnings(record=True) as caught_warnings:
            BooleanValue.new(
                "False", ":booleanProp", permissions=Permissions.PUBLIC, resource_id="res_id", comment=None
            )
        assert len(caught_warnings) == 0

    def test_good_comment_is_nan(self) -> None:
        with warnings.catch_warnings(record=True) as caught_warnings:
            val = BooleanValue.new(
                "False",
                ":booleanProp",
                permissions=Permissions.PUBLIC,
                resource_id="res_id",
                comment=pd.NA,  # type: ignore[arg-type]
            )
        assert len(caught_warnings) == 0
        assert val.comment is None

    def test_warns(self) -> None:
        with pytest.warns(XmllibInputWarning):
            BooleanValue.new(
                "other", ":booleanProp", permissions=Permissions.PUBLIC, resource_id="res_id", comment=None
            )

    def test_warns_potentially_empty_comment(self) -> None:
        with pytest.warns(XmllibInputInfo):
            BooleanValue.new(
                True, ":booleanProp", permissions=Permissions.PUBLIC, resource_id="res_id", comment=str(pd.NA)
            )


class TestColorValue:
    def test_good(self) -> None:
        with warnings.catch_warnings(record=True) as caught_warnings:
            ColorValue.new("#FFFFFF", ":colorProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None)
        assert len(caught_warnings) == 0

    def test_warns(self) -> None:
        with pytest.warns(XmllibInputWarning):
            ColorValue.new(
                "invalidColor", ":colorProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None
            )


class TestDateValue:
    def test_good(self) -> None:
        with warnings.catch_warnings(record=True) as caught_warnings:
            DateValue.new("2023-01-01", ":dateProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None)
        assert len(caught_warnings) == 0

    def test_warns(self) -> None:
        with pytest.warns(XmllibInputWarning):
            DateValue.new(
                "invalidDate", ":dateProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None
            )


class TestDecimalValue:
    def test_good(self) -> None:
        with warnings.catch_warnings(record=True) as caught_warnings:
            DecimalValue.new("3.14", ":decimalProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None)
        assert len(caught_warnings) == 0

    def test_warns(self) -> None:
        with pytest.warns(XmllibInputWarning):
            DecimalValue.new(
                "invalidDecimal", ":decimalProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None
            )


class TestGeonameValue:
    def test_good(self) -> None:
        with warnings.catch_warnings(record=True) as caught_warnings:
            GeonameValue.new(
                "00099", ":geonameProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None
            )
        assert len(caught_warnings) == 0

    def test_warns(self) -> None:
        with pytest.warns(XmllibInputWarning):
            GeonameValue.new(
                "invalidGeoname", ":geonameProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None
            )


class TestIntValue:
    def test_good(self) -> None:
        with warnings.catch_warnings(record=True) as caught_warnings:
            IntValue.new("42", ":intProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None)
        assert len(caught_warnings) == 0

    def test_warns(self) -> None:
        with pytest.warns(XmllibInputWarning):
            IntValue.new("invalidInt", ":intProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None)


class TestLinkValue:
    def test_good(self) -> None:
        with warnings.catch_warnings(record=True) as caught_warnings:
            LinkValue.new("link", ":linkProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None)
        assert len(caught_warnings) == 0

    def test_warns(self) -> None:
        with pytest.warns(XmllibInputWarning):
            LinkValue.new(None, ":linkProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None)


class TestListValue:
    def test_good(self) -> None:
        with warnings.catch_warnings(record=True) as caught_warnings:
            ListValue.new(
                "item1", "listName", ":listProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None
            )
        assert len(caught_warnings) == 0

    def test_good_with_list_iri_and_none(self) -> None:
        with warnings.catch_warnings(record=True) as caught_warnings:
            ListValue.new(
                "http://rdfh.ch/lists/0121/eDaD2IJrR3-zRF1BZnlglw",
                None,
                ":listProp",
                resource_id="res_id",
                permissions=Permissions.PUBLIC,
                comment=None,
            )
        assert len(caught_warnings) == 0

    def test_warns_false_node(self) -> None:
        with pytest.warns(XmllibInputWarning):
            ListValue.new(None, "list", ":listProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None)

    def test_warns_false_list(self) -> None:
        with pytest.warns(XmllibInputWarning):
            ListValue.new("Node", None, ":listProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None)


class TestRichtext:
    def test_good(self) -> None:
        with warnings.catch_warnings(record=True) as caught_warnings:
            Richtext.new(
                "<p>Hello World</p>",
                ":richtextProp",
                resource_id="res_id",
                permissions=Permissions.PUBLIC,
                comment=None,
            )
        assert len(caught_warnings) == 0

    def test_warns(self) -> None:
        with pytest.warns(XmllibInputWarning):
            Richtext.new(None, ":richtextProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None)

    def test_raises(self) -> None:
        msg = regex.escape(
            "The entered richtext value could not be converted to a valid XML.\n"
            "Original error message: Opening and ending tag mismatch: p line 1 and ignore-this, line 1, column 43\n"
            "Potential line/column numbers are relative to this text: <ignore-this><p> not escaped</ignore-this>"
        )
        with pytest.raises(InputError, match=msg):
            Richtext.new(
                "<p> not escaped", ":richtextProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None
            )


class TestSimpleText:
    def test_good(self) -> None:
        with warnings.catch_warnings(record=True) as caught_warnings:
            SimpleText.new(
                "Hello World", ":simpleTextProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None
            )
        assert len(caught_warnings) == 0

    def test_warns(self) -> None:
        with pytest.warns(XmllibInputWarning):
            SimpleText.new(None, ":simpleTextProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None)


class TestTimeValue:
    def test_good(self) -> None:
        with warnings.catch_warnings(record=True) as caught_warnings:
            TimeValue.new(
                "2009-10-10T12:00:00-05:00",
                ":timeProp",
                resource_id="res_id",
                permissions=Permissions.PUBLIC,
                comment=None,
            )
        assert len(caught_warnings) == 0

    def test_warns(self) -> None:
        with pytest.warns(XmllibInputWarning):
            TimeValue.new(
                "invalidTime", ":timeProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None
            )


class TestUriValue:
    def test_good(self) -> None:
        with warnings.catch_warnings(record=True) as caught_warnings:
            UriValue.new(
                "https://example.com", ":uriProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None
            )
        assert len(caught_warnings) == 0

    def test_warns(self) -> None:
        with pytest.warns(XmllibInputWarning):
            UriValue.new("invalidUri", ":uriProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None)


if __name__ == "__main__":
    pytest.main([__file__])



from rdflib import Literal
from rdflib import URIRef

from dsp_tools.commands.validate_data.models.input_problems import UnknownClassesInData
from dsp_tools.commands.validate_data.models.validation import RDFGraphs
from dsp_tools.commands.validate_data.utils import reformat_onto_iri
from dsp_tools.utils.rdflib_constants import KNORA_API_STR


def check_for_unknown_resource_classes(
    rdf_graphs: RDFGraphs, used_resource_iris: set[str]
) -> UnknownClassesInData | None:
    """
    Checks if any classes are referenced in the data that are not in the ontology.

    Args:
        rdf_graphs: Data graphs
        used_resource_iris: resource IRIs in use

    Returns:
        Unknown classes if any
    """
    res_cls = _get_all_onto_classes(rdf_graphs)
    if extra_cls := used_resource_iris - res_cls:
        unknown_classes = {reformat_onto_iri(x) for x in extra_cls}
        defined_classes = {reformat_onto_iri(x) for x in res_cls}
        return UnknownClassesInData(unknown_classes=unknown_classes, defined_classes=defined_classes)
    return None


def _get_all_onto_classes(rdf_graphs: RDFGraphs) -> set[str]:
    ontos = rdf_graphs.ontos + rdf_graphs.knora_api
    is_resource_iri = URIRef(KNORA_API_STR + "isResourceClass")
    resource_classes = set(ontos.subjects(is_resource_iri, Literal(True)))
    is_usable = URIRef(KNORA_API_STR + "canBeInstantiated")
    usable_resource_classes = set(ontos.subjects(is_usable, Literal(True)))
    user_facing = usable_resource_classes.intersection(resource_classes)
    return {str(x) for x in user_facing}


def get_msg_str_unknown_classes_in_data(unknown: UnknownClassesInData) -> str:
    if unknown_onto_msg := _get_unknown_ontos_msg(unknown):
        return unknown_onto_msg
    unknown_classes = sorted(list(unknown.unknown_classes))
    known_classes = sorted(list(unknown.defined_classes))
    return (
        f"Your data uses resource classes that do not exist in the ontologies in the database.\n"
        f"The following classes that are used in the data are unknown: {', '.join(unknown_classes)}\n"
        f"The following classes exist in the uploaded ontologies: {', '.join(known_classes)}"
    )


def _get_unknown_ontos_msg(unknown: UnknownClassesInData) -> str | None:
    def split_prefix(relative_iri: str) -> str | None:
        if ":" not in relative_iri:
            return None
        return relative_iri.split(":")[0]

    used_ontos = set(not_knora for x in unknown.unknown_classes if (not_knora := split_prefix(x)))
    exising_ontos = set(not_knora for x in unknown.defined_classes if (not_knora := split_prefix(x)))
    if unknown_found := used_ontos - exising_ontos:
        return (
            f"Your data uses ontologies that don't exist in the database.\n"
            f"The following ontologies that are used in the data are unknown: {', '.join(unknown_found)}\n"
            f"The following ontologies are uploaded: {', '.join(exising_ontos)}"
        )
    return None



import pickle
import sys

from loguru import logger

from dsp_tools.cli.args import ServerCredentials
from dsp_tools.clients.authentication_client_live import AuthenticationClientLive
from dsp_tools.clients.connection_live import ConnectionLive
from dsp_tools.clients.legal_info_client import LegalInfoClient
from dsp_tools.clients.legal_info_client_live import LegalInfoClientLive
from dsp_tools.commands.xmlupload.models.ingest import AssetClient
from dsp_tools.commands.xmlupload.models.ingest import BulkIngestedAssetClient
from dsp_tools.commands.xmlupload.models.ingest import DspIngestClientLive
from dsp_tools.commands.xmlupload.models.upload_clients import UploadClients
from dsp_tools.commands.xmlupload.models.upload_state import UploadState
from dsp_tools.commands.xmlupload.prepare_xml_input.list_client import ListClient
from dsp_tools.commands.xmlupload.prepare_xml_input.list_client import ListClientLive
from dsp_tools.commands.xmlupload.project_client import ProjectClient
from dsp_tools.commands.xmlupload.project_client import ProjectClientLive
from dsp_tools.commands.xmlupload.upload_config import UploadConfig
from dsp_tools.commands.xmlupload.xmlupload import execute_upload
from dsp_tools.utils.ansi_colors import RED
from dsp_tools.utils.ansi_colors import RESET_TO_DEFAULT


def resume_xmlupload(creds: ServerCredentials, skip_first_resource: bool = False) -> bool:
    """
    Resume an interrupted xmlupload.

    Args:
        creds: credentials to access the DSP server
        skip_first_resource: if this flag is set, the first resource of the pending resources is removed

    Returns:
        True if all resources could be uploaded without errors; False if one of the resources could not be
        uploaded because there is an error in it
    """
    server = creds.server
    upload_state = _read_upload_state_from_disk(server)
    if skip_first_resource:
        _skip_first_resource(upload_state)

    _print_and_log(upload_state, server)

    auth = AuthenticationClientLive(server, creds.user, creds.password)
    con = ConnectionLive(server, auth)

    ingest_client: AssetClient
    if upload_state.config.media_previously_uploaded:
        ingest_client = BulkIngestedAssetClient()
    else:
        ingest_client = DspIngestClientLive(creds.dsp_ingest_url, auth, upload_state.config.shortcode, ".")

    project_client: ProjectClient = ProjectClientLive(con, upload_state.config.shortcode)
    list_client: ListClient = ListClientLive(con, project_client.get_project_iri())
    legal_info_client: LegalInfoClient = LegalInfoClientLive(server, upload_state.config.shortcode, auth)
    clients = UploadClients(ingest_client, project_client, list_client, legal_info_client)

    return execute_upload(clients, upload_state)


def _read_upload_state_from_disk(server: str) -> UploadState:
    save_location = UploadConfig().with_server_info(server, "foo").diagnostics.save_location
    with open(save_location, "rb") as f:
        saved_state: UploadState = pickle.load(f)  # noqa: S301 (deserialization of untrusted data)
    return saved_state


def _skip_first_resource(upload_state: UploadState) -> None:
    if len(upload_state.pending_resources) > 0:
        upload_state.pending_resources.pop(0)
    else:
        msg = (
            "The list of pending resources is empty.\n"
            "It is not yet possible to skip the first item of the stashed properties.\n"
            "Do you want to continue with the upload of the stashed properties anyway? [y/n]"
        )
        resp = None
        while resp not in ["y", "n"]:
            resp = input(RED + msg + RESET_TO_DEFAULT)
        if resp == "n":
            sys.exit(1)


def _print_and_log(upload_state: UploadState, server: str) -> None:
    previous_successful = len(upload_state.iri_resolver.lookup)
    previous_failed = len(upload_state.failed_uploads)
    previous_total = previous_successful + previous_failed
    msg = (
        f"Resuming upload for project {upload_state.config.shortcode} on server {server}. "
        f"Number of resources uploaded until now: {previous_total}"
    )
    if previous_failed:
        msg += f" ({previous_failed} of them failed)"
    logger.info(msg)
    print("\n==========================\n" + msg + "\n==========================\n")



import argparse
from pathlib import Path

from loguru import logger

from dsp_tools.cli.args import ServerCredentials
from dsp_tools.cli.args import ValidationSeverity
from dsp_tools.commands.excel2json.lists.make_lists import excel2lists
from dsp_tools.commands.excel2json.old_lists import old_excel2lists
from dsp_tools.commands.excel2json.old_lists import validate_lists_section_with_schema
from dsp_tools.commands.excel2json.project import excel2json
from dsp_tools.commands.excel2json.project import old_excel2json
from dsp_tools.commands.excel2json.properties import excel2properties
from dsp_tools.commands.excel2json.resources import excel2resources
from dsp_tools.commands.excel2xml.excel2xml_cli import excel2xml
from dsp_tools.commands.id2iri import id2iri
from dsp_tools.commands.ingest_xmlupload.create_resources.upload_xml import ingest_xmlupload
from dsp_tools.commands.ingest_xmlupload.ingest_files.ingest_files import ingest_files
from dsp_tools.commands.ingest_xmlupload.upload_files.upload_files import upload_files
from dsp_tools.commands.project.create.project_create_all import create_project
from dsp_tools.commands.project.create.project_create_lists import create_only_lists
from dsp_tools.commands.project.create.project_validate import validate_project
from dsp_tools.commands.project.get import get_project
from dsp_tools.commands.resume_xmlupload.resume_xmlupload import resume_xmlupload
from dsp_tools.commands.rosetta import upload_rosetta
from dsp_tools.commands.start_stack import StackConfiguration
from dsp_tools.commands.start_stack import StackHandler
from dsp_tools.commands.template import generate_template_repo
from dsp_tools.commands.validate_data.validate_data import validate_data
from dsp_tools.commands.xmlupload.upload_config import UploadConfig
from dsp_tools.commands.xmlupload.xmlupload import xmlupload
from dsp_tools.error.exceptions import InputError
from dsp_tools.utils.xml_parsing.parse_clean_validate_xml import parse_and_validate_xml_file


def call_requested_action(args: argparse.Namespace) -> bool:  # noqa: PLR0912 (too many branches)
    """
    Call the appropriate function of DSP-TOOLS.

    Args:
        args: parsed CLI arguments

    Raises:
        BaseError: from the called function
        InputError: from the called function
        unexpected errors from the called methods and underlying libraries

    Returns:
        success status
    """
    match args.action:
        case "create":
            result = _call_create(args)
        case "xmlupload":
            result = _call_xmlupload(args)
        case "validate-data":
            result = _call_validate_data(args)
        case "resume-xmlupload":
            result = _call_resume_xmlupload(args)
        case "excel2json":
            result = _call_excel2json(args)
        case "old-excel2json":
            result = _call_old_excel2json(args)
        case "excel2lists":
            result = _call_excel2lists(args)
        case "old-excel2lists":
            result = _call_old_excel2lists(args)
        case "excel2resources":
            result = _call_excel2resources(args)
        case "excel2properties":
            result = _call_excel2properties(args)
        case "id2iri":
            result = _call_id2iri(args)
        case "excel2xml":
            result = _call_excel2xml(args)
        case "start-stack":
            result = _call_start_stack(args)
        case "stop-stack":
            result = _call_stop_stack()
        case "get":
            result = _call_get(args)
        case "upload-files":
            result = _call_upload_files(args)
        case "ingest-files":
            result = _call_ingest_files(args)
        case "ingest-xmlupload":
            result = _call_ingest_xmlupload(args)
        case "template":
            result = generate_template_repo()
        case "rosetta":
            result = upload_rosetta()
        case _:
            print(f"ERROR: Unknown action '{args.action}'")
            logger.error(f"Unknown action '{args.action}'")
            result = False
    return result


def _call_stop_stack() -> bool:
    stack_handler = StackHandler(StackConfiguration())
    return stack_handler.stop_stack()


def _call_start_stack(args: argparse.Namespace) -> bool:
    stack_handler = StackHandler(
        StackConfiguration(
            max_file_size=args.max_file_size,
            enforce_docker_system_prune=args.prune,
            suppress_docker_system_prune=args.no_prune,
            latest_dev_version=args.latest,
            upload_test_data=args.with_test_data,
            custom_host=args.custom_host,
        )
    )
    return stack_handler.start_stack()


def _call_excel2xml(args: argparse.Namespace) -> bool:
    success, _ = excel2xml(
        datafile=args.data_source,
        shortcode=args.project_shortcode,
        default_ontology=args.ontology_name,
    )
    return success


def _call_id2iri(args: argparse.Namespace) -> bool:
    return id2iri(
        xml_file=args.xmlfile,
        json_file=args.mapping,
        remove_resource_if_id_in_mapping=args.remove_resources,
    )


def _call_excel2properties(args: argparse.Namespace) -> bool:
    _, success = excel2properties(
        excelfile=args.excelfile,
        path_to_output_file=args.properties_section,
    )
    return success


def _call_excel2resources(args: argparse.Namespace) -> bool:
    _, success = excel2resources(
        excelfile=args.excelfile,
        path_to_output_file=args.resources_section,
    )
    return success


def _call_old_excel2lists(args: argparse.Namespace) -> bool:
    _, success = old_excel2lists(
        excelfolder=args.excelfolder,
        path_to_output_file=args.lists_section,
        verbose=args.verbose,
    )
    return success


def _call_excel2lists(args: argparse.Namespace) -> bool:
    _, success = excel2lists(
        excelfolder=args.excelfolder,
        path_to_output_file=args.lists_section,
    )
    return success


def _call_excel2json(args: argparse.Namespace) -> bool:
    return excel2json(
        data_model_files=args.excelfolder,
        path_to_output_file=args.project_definition,
    )


def _call_old_excel2json(args: argparse.Namespace) -> bool:
    return old_excel2json(
        data_model_files=args.excelfolder,
        path_to_output_file=args.project_definition,
    )


def _call_upload_files(args: argparse.Namespace) -> bool:
    return upload_files(
        xml_file=Path(args.xml_file),
        creds=_get_creds(args),
        imgdir=Path(args.imgdir),
    )


def _call_ingest_files(args: argparse.Namespace) -> bool:
    return ingest_files(creds=_get_creds(args), shortcode=args.shortcode)


def _call_ingest_xmlupload(args: argparse.Namespace) -> bool:
    interrupt_after = args.interrupt_after if args.interrupt_after > 0 else None
    return ingest_xmlupload(
        xml_file=Path(args.xml_file),
        creds=_get_creds(args),
        interrupt_after=interrupt_after,
        skip_validation=args.skip_validation,
    )


def _call_xmlupload(args: argparse.Namespace) -> bool:
    if args.validate_only:
        success = parse_and_validate_xml_file(Path(args.xmlfile))
        print("The XML file is syntactically correct.")
        return success
    else:
        interrupt_after = args.interrupt_after if args.interrupt_after > 0 else None
        match args.validation_severity:
            case "info":
                severity = ValidationSeverity.INFO
            case "warning":
                severity = ValidationSeverity.WARNING
            case "error":
                severity = ValidationSeverity.ERROR
            case _:
                raise InputError(
                    f"The entered validation severity '{args.validation_severity}' "
                    f"is not part of the allowed values: info, warning, error."
                )
        return xmlupload(
            input_file=Path(args.xmlfile),
            creds=_get_creds(args),
            imgdir=args.imgdir,
            config=UploadConfig(
                interrupt_after=interrupt_after,
                skip_iiif_validation=args.no_iiif_uri_validation,
                skip_validation=args.skip_validation,
                ignore_duplicate_files_warning=args.ignore_duplicate_files_warning,
                validation_severity=severity,
            ),
        )


def _call_validate_data(args: argparse.Namespace) -> bool:
    return validate_data(
        filepath=Path(args.xmlfile),
        creds=_get_creds(args),
        save_graphs=args.save_graphs,
        ignore_duplicate_files_warning=args.ignore_duplicate_files_warning,
    )


def _call_resume_xmlupload(args: argparse.Namespace) -> bool:
    return resume_xmlupload(
        creds=_get_creds(args),
        skip_first_resource=args.skip_first_resource,
    )


def _call_get(args: argparse.Namespace) -> bool:
    return get_project(
        project_identifier=args.project,
        outfile_path=args.project_definition,
        creds=_get_creds(args),
        verbose=args.verbose,
    )


def _call_create(args: argparse.Namespace) -> bool:
    success = False
    match args.lists_only, args.validate_only:
        case True, True:
            success = validate_lists_section_with_schema(args.project_definition)
            print("'Lists' section of the JSON project file is syntactically correct and passed validation.")
        case True, False:
            _, success = create_only_lists(
                project_file_as_path_or_parsed=args.project_definition,
                creds=_get_creds(args),
            )
        case False, True:
            success = validate_project(args.project_definition)
            print("JSON project file is syntactically correct and passed validation.")
        case False, False:
            success = create_project(
                project_file_as_path_or_parsed=args.project_definition,
                creds=_get_creds(args),
                verbose=args.verbose,
            )
    return success


def _get_creds(args: argparse.Namespace) -> ServerCredentials:
    return ServerCredentials(
        server=args.server,
        user=args.user,
        password=args.password,
        dsp_ingest_url=args.dsp_ingest_url,
    )



from __future__ import annotations

from rdflib import XSD

from dsp_tools.commands.xmlupload.models.processed.values import ProcessedBoolean
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedColor
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedDecimal
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedGeometry
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedGeoname
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedInt
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedSimpleText
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedTime
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedUri
from dsp_tools.commands.xmlupload.models.rdf_models import RDFPropTypeInfo
from dsp_tools.utils.rdflib_constants import KNORA_API
from dsp_tools.utils.xml_parsing.models.parsed_resource import KnoraValueType

# values that need special considerations during the graph construction
LIST_PROP_TYPE_INFO = RDFPropTypeInfo(KNORA_API.ListValue, KNORA_API.listValueAsListNode)
LINK_PROP_TYPE_INFO = RDFPropTypeInfo(KNORA_API.LinkValue, KNORA_API.linkValueHasTargetIri)
RICHTEXT_PROP_TYPE_INFO = RDFPropTypeInfo(KNORA_API.TextValue, KNORA_API.textValueAsXml, XSD.string)

# values suitable for the generic lookup
BOOLEAN_PROP_TYPE_INFO = RDFPropTypeInfo(KNORA_API.BooleanValue, KNORA_API.booleanValueAsBoolean, XSD.boolean)
COLOR_PROP_TYPE_INFO = RDFPropTypeInfo(KNORA_API.ColorValue, KNORA_API.colorValueAsColor, XSD.string)
DECIMAL_PROP_TYPE_INFO = RDFPropTypeInfo(KNORA_API.DecimalValue, KNORA_API.decimalValueAsDecimal, XSD.decimal)
GEOMETRY_PROP_TYPE_INFO = RDFPropTypeInfo(KNORA_API.GeomValue, KNORA_API.geometryValueAsGeometry, XSD.string)
GEONAME_PROP_TYPE_INFO = RDFPropTypeInfo(KNORA_API.GeonameValue, KNORA_API.geonameValueAsGeonameCode, XSD.string)
INT_PROP_TYPE_INFO = RDFPropTypeInfo(KNORA_API.IntValue, KNORA_API.intValueAsInt, XSD.integer)
SIMPLE_TEXT_PROP_TYPE_INFO = RDFPropTypeInfo(KNORA_API.TextValue, KNORA_API.valueAsString, XSD.string)
TIME_PROP_TYPE_INFO = RDFPropTypeInfo(KNORA_API.TimeValue, KNORA_API.timeValueAsTimeStamp, XSD.dateTimeStamp)
URI_PROP_TYPE_INFO = RDFPropTypeInfo(KNORA_API.UriValue, KNORA_API.uriValueAsUri, XSD.anyURI)

RDF_LITERAL_PROP_TYPE_MAPPER = {
    ProcessedBoolean: BOOLEAN_PROP_TYPE_INFO,
    ProcessedColor: COLOR_PROP_TYPE_INFO,
    ProcessedDecimal: DECIMAL_PROP_TYPE_INFO,
    ProcessedGeometry: GEOMETRY_PROP_TYPE_INFO,
    ProcessedGeoname: GEONAME_PROP_TYPE_INFO,
    ProcessedInt: INT_PROP_TYPE_INFO,
    ProcessedSimpleText: SIMPLE_TEXT_PROP_TYPE_INFO,
    ProcessedTime: TIME_PROP_TYPE_INFO,
    ProcessedUri: URI_PROP_TYPE_INFO,
}

# file values
ARCHIVE_FILE_VALUE = RDFPropTypeInfo(KNORA_API.ArchiveFileValue, KNORA_API.hasArchiveFileValue, XSD.string)
AUDIO_FILE_VALUE = RDFPropTypeInfo(KNORA_API.AudioFileValue, KNORA_API.hasAudioFileValue, XSD.string)
DOCUMENT_FILE_VALUE = RDFPropTypeInfo(KNORA_API.DocumentFileValue, KNORA_API.hasDocumentFileValue, XSD.string)
MOVING_IMAGE_FILE_VALUE = RDFPropTypeInfo(KNORA_API.MovingImageFileValue, KNORA_API.hasMovingImageFileValue, XSD.string)
STILL_IMAGE_FILE_VALUE = RDFPropTypeInfo(KNORA_API.StillImageFileValue, KNORA_API.hasStillImageFileValue, XSD.string)
TEXT_FILE_VALUE = RDFPropTypeInfo(KNORA_API.TextFileValue, KNORA_API.hasTextFileValue, XSD.string)
IIIF_URI_VALUE = RDFPropTypeInfo(KNORA_API.StillImageExternalFileValue, KNORA_API.hasStillImageFileValue, XSD.anyURI)


FILE_TYPE_TO_RDF_MAPPER = {
    KnoraValueType.ARCHIVE_FILE: ARCHIVE_FILE_VALUE,
    KnoraValueType.AUDIO_FILE: AUDIO_FILE_VALUE,
    KnoraValueType.DOCUMENT_FILE: DOCUMENT_FILE_VALUE,
    KnoraValueType.MOVING_IMAGE_FILE: MOVING_IMAGE_FILE_VALUE,
    KnoraValueType.STILL_IMAGE_FILE: STILL_IMAGE_FILE_VALUE,
    KnoraValueType.TEXT_FILE: TEXT_FILE_VALUE,
}



import contextlib
from pathlib import Path

import pytest

from dsp_tools.commands.xmlupload.upload_config import UploadConfig


def test_save_location() -> None:
    server = "dasch.swiss"
    expected_path = Path(Path.home() / f".dsp-tools/xmluploads/{server}/resumable/latest.pkl")
    config_with_save_location = UploadConfig().with_server_info(server=server, shortcode="1234")
    diagnostics = config_with_save_location.diagnostics
    assert expected_path == diagnostics.save_location
    # in case these folders didn't exist before: tidy up
    with contextlib.suppress(OSError):
        diagnostics.save_location.parent.rmdir()  # raises OSError if the folder is not empty
        diagnostics.save_location.parent.parent.rmdir()


if __name__ == "__main__":
    pytest.main([__file__])



# mypy: disable-error-code="no-untyped-def"
import os
from unittest import mock

import pytest
from lxml import etree

from dsp_tools.xmllib.internal.serialise_values import _sort_and_group_values
from dsp_tools.xmllib.internal.serialise_values import serialise_values
from dsp_tools.xmllib.models.config_options import Permissions
from dsp_tools.xmllib.models.internal.values import BooleanValue
from dsp_tools.xmllib.models.internal.values import ColorValue
from dsp_tools.xmllib.models.internal.values import DateValue
from dsp_tools.xmllib.models.internal.values import DecimalValue
from dsp_tools.xmllib.models.internal.values import GeonameValue
from dsp_tools.xmllib.models.internal.values import IntValue
from dsp_tools.xmllib.models.internal.values import LinkValue
from dsp_tools.xmllib.models.internal.values import ListValue
from dsp_tools.xmllib.models.internal.values import Richtext
from dsp_tools.xmllib.models.internal.values import SimpleText
from dsp_tools.xmllib.models.internal.values import TimeValue
from dsp_tools.xmllib.models.internal.values import UriValue
from dsp_tools.xmllib.models.internal.values import Value


@pytest.fixture
def mixed_values() -> list[Value]:
    return [
        BooleanValue("false", ":bool"),
        LinkValue("b", ":link"),
        IntValue("2", ":int"),
        LinkValue("a", ":link"),
        IntValue("1", ":int"),
        SimpleText("a", ":text1"),
        SimpleText("b", ":text2"),
    ]


EXPECTED_TYPE_LOOKUP = {
    ":bool": "boolean",
    ":int": "integer",
    ":text1": "simpletext",
    ":text2": "simpletext",
    ":link": "resptr",
}


class TestSortValue:
    def test_no_sorting(self, mixed_values):
        result_tups, type_lookup = _sort_and_group_values(mixed_values)
        assert type_lookup == EXPECTED_TYPE_LOOKUP
        expected_props = {x.prop_name for x in mixed_values}
        actual_props = {x[0] for x in result_tups}
        assert len(result_tups) == len(expected_props)
        assert expected_props == actual_props

    @mock.patch.dict(os.environ, {"XMLLIB_SORT_PROPERTIES": "true"})
    def test_with_sorting(self, mixed_values):
        result_tups, type_lookup = _sort_and_group_values(mixed_values)
        assert type_lookup == EXPECTED_TYPE_LOOKUP
        expected_prop_order = [":bool", ":int", ":link", ":text1", ":text2"]
        result_order = [x[0] for x in result_tups]
        assert result_order == expected_prop_order
        bool_val = result_tups[0][1]
        assert len(bool_val) == 1
        int_val = result_tups[1][1]
        assert len(int_val) == 2
        assert int_val[0].value == "1"
        assert int_val[1].value == "2"
        link_val = result_tups[2][1]
        assert len(link_val) == 2
        assert link_val[0].value == "a"
        assert link_val[1].value == "b"
        text_1 = result_tups[3][1]
        assert len(text_1) == 1
        text_2 = result_tups[4][1]
        assert len(text_2) == 1


class TestSerialiseValues:
    def test_boolean(self):
        v: list[Value] = [
            BooleanValue.new("0", ":booleanProp", resource_id="res_id", permissions=Permissions.PUBLIC, comment=None)
        ]
        result = serialise_values(v)
        assert len(result) == 1
        expected = (
            b"<boolean-prop "
            b'xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":booleanProp">'
            b'<boolean permissions="public">false</boolean>'
            b"</boolean-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected

    def test_color(self):
        v: list[Value] = [ColorValue("#000000", ":colorProp", permissions=Permissions.PUBLIC)]
        result = serialise_values(v)
        assert len(result) == 1
        expected = (
            b"<color-prop "
            b'xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":colorProp">'
            b'<color permissions="public">#000000</color>'
            b"</color-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected

    def test_date(self):
        v: list[Value] = [DateValue("2023-01-01", ":dateProp", permissions=Permissions.PUBLIC)]
        result = serialise_values(v)
        assert len(result) == 1
        expected = (
            b"<date-prop "
            b'xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":dateProp">'
            b'<date permissions="public">2023-01-01</date>'
            b"</date-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected

    def test_decimal(self):
        v: list[Value] = [DecimalValue("3.14", ":decimalProp", permissions=Permissions.PUBLIC)]
        result = serialise_values(v)
        assert len(result) == 1
        expected = (
            b"<decimal-prop "
            b'xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":decimalProp">'
            b'<decimal permissions="public">3.14</decimal>'
            b"</decimal-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected

    def test_geoname(self):
        v: list[Value] = [GeonameValue("99", ":geonameProp", permissions=Permissions.PUBLIC)]
        result = serialise_values(v)
        assert len(result) == 1
        expected = (
            b"<geoname-prop "
            b'xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":geonameProp">'
            b'<geoname permissions="public">99</geoname>'
            b"</geoname-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected

    def test_int(self):
        v: list[Value] = [IntValue("42", ":intProp", permissions=Permissions.PUBLIC)]
        result = serialise_values(v)
        assert len(result) == 1
        expected = (
            b"<integer-prop "
            b'xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":intProp">'
            b'<integer permissions="public">42</integer>'
            b"</integer-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected

    def test_link(self):
        v: list[Value] = [LinkValue("res_link", ":linkProp", permissions=Permissions.PUBLIC)]
        result = serialise_values(v)
        assert len(result) == 1
        expected = (
            b"<resptr-prop "
            b'xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":linkProp">'
            b'<resptr permissions="public">res_link</resptr>'
            b"</resptr-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected

    def test_list(self):
        v: list[Value] = [ListValue("item1", "listName", ":listProp", permissions=Permissions.PUBLIC)]
        result = serialise_values(v)
        assert len(result) == 1
        expected = (
            b"<list-prop "
            b'xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":listProp" list="listName">'
            b'<list permissions="public">item1</list>'
            b"</list-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected

    @pytest.mark.parametrize(
        ("orig", "expected"),
        [
            (
                "<strong>standard standoff tag</strong><unsupported>Hello World</unsupported>",
                "<strong>standard standoff tag</strong>&lt;unsupported&gt;Hello World&lt;/unsupported&gt;",
            ),
            ("&amp; &lt; &gt;", "&amp; &lt; &gt;"),
            ("'uuas\\. 11` \\a\\ i! 1 ?7 Rinne   \\Rinne", "'uuas\\. 11` \\a\\ i! 1 ?7 Rinne   \\Rinne"),
            ("1 < 2 & 4 > 3", "1 &lt; 2 &amp; 4 &gt; 3"),
        ],
    )
    def test_richtext_tags(self, orig: str, expected: str):
        result = serialise_values([Richtext(orig, ":richtextProp")])
        assert len(result) == 1
        expected_xml = (
            '<text-prop xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            'name=":richtextProp">'
            f'<text encoding="xml">{expected}</text></text-prop>'
        )
        res_str = etree.tostring(result.pop(0), encoding="unicode")
        assert res_str == expected_xml

    def test_simpletext(self):
        v: list[Value] = [SimpleText("Hello World", ":simpleTextProp", permissions=Permissions.PUBLIC)]
        result = serialise_values(v)
        assert len(result) == 1
        expected = (
            b"<text-prop "
            b'xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":simpleTextProp">'
            b'<text permissions="public" encoding="utf8">Hello World</text>'
            b"</text-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected

    def test_simpletext_escapes(self):
        original = "'uuas\\. 11` \\a\\ i! 1 ?7 Rinne   \\Rinne"
        expected = "'uuas\\. 11` \\a\\ i! 1 ?7 Rinne   \\Rinne"
        result = serialise_values([SimpleText(original, ":simpleTextProp")])
        assert len(result) == 1
        expected_xml = (
            '<text-prop xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            'name=":simpleTextProp">'
            f'<text encoding="utf8">{expected}</text></text-prop>'
        )
        res_str = str(etree.tostring(result.pop(0)), encoding="utf-8")
        assert res_str == expected_xml

    def test_time(self):
        v: list[Value] = [TimeValue("2009-10-10T12:00:00-05:00", ":timeProp", permissions=Permissions.PUBLIC)]
        result = serialise_values(v)
        assert len(result) == 1
        expected = (
            b"<time-prop "
            b'xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":timeProp">'
            b'<time permissions="public">2009-10-10T12:00:00-05:00</time>'
            b"</time-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected

    def test_uri(self):
        v: list[Value] = [UriValue("https://example.com", ":uriProp", permissions=Permissions.PUBLIC)]
        result = serialise_values(v)
        assert len(result) == 1
        expected = (
            b"<uri-prop "
            b'xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":uriProp">'
            b'<uri permissions="public">https://example.com</uri>'
            b"</uri-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected

    def test_value_with_default_permission(self):
        v: list[Value] = [BooleanValue("false", ":booleanProp")]
        result = serialise_values(v)
        assert len(result) == 1
        expected = (
            b"<boolean-prop "
            b'xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":booleanProp">'
            b"<boolean>false</boolean>"
            b"</boolean-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected

    def test_several_values_different_generic_property(self):
        v: list[Value] = [
            LinkValue("res_link", ":linkProp"),
            SimpleText("Hello World", ":simpleTextProp"),
        ]
        result = serialise_values(v)
        assert len(result) == 2
        expected = (
            b"<resptr-prop "
            b'xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":linkProp"><resptr>res_link</resptr>'
            b"</resptr-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected
        expected = (
            b"<text-prop "
            b'xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":simpleTextProp">'
            b'<text encoding="utf8">Hello World</text>'
            b"</text-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected

    def test_several_values_different_property(self):
        v: list[Value] = [
            BooleanValue("false", ":booleanProp"),
            LinkValue("res_link", ":linkProp"),
        ]
        result = serialise_values(v)
        assert len(result) == 2
        expected = (
            b'<boolean-prop xmlns="https://dasch.swiss/schema" '
            b'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":booleanProp">'
            b"<boolean>false</boolean>"
            b"</boolean-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected
        expected = (
            b"<resptr-prop "
            b'xmlns="https://dasch.swiss/schema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":linkProp">'
            b"<resptr>res_link</resptr>"
            b"</resptr-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected

    def test_several_values_same_property(self):
        v: list[Value] = [
            LinkValue("open_permission", ":linkProp", permissions=Permissions.PUBLIC),
            LinkValue("default_permission", ":linkProp"),
        ]
        result = serialise_values(v)
        assert len(result) == 1
        expected = (
            b'<resptr-prop xmlns="https://dasch.swiss/schema" '
            b'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
            b'name=":linkProp">'
            b'<resptr permissions="public">open_permission</resptr>'
            b"<resptr>default_permission</resptr>"
            b"</resptr-prop>"
        )
        res_str = etree.tostring(result.pop(0))
        assert res_str == expected


if __name__ == "__main__":
    pytest.main([__file__])



# mypy: disable-error-code="method-assign,no-untyped-def"
import pytest
import regex

from dsp_tools.commands.xmlupload.models.lookup_models import XmlReferenceLookups
from dsp_tools.commands.xmlupload.models.permission import Permissions
from dsp_tools.commands.xmlupload.models.permission import PermissionValue
from dsp_tools.commands.xmlupload.models.processed.file_values import ProcessedFileValue
from dsp_tools.commands.xmlupload.models.processed.file_values import ProcessedIIIFUri
from dsp_tools.commands.xmlupload.models.processed.res import MigrationMetadata
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedBoolean
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedColor
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedDate
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedDecimal
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedGeometry
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedGeoname
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedInt
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedInterval
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedLink
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedList
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedRichtext
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedSimpleText
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedTime
from dsp_tools.commands.xmlupload.models.processed.values import ProcessedUri
from dsp_tools.commands.xmlupload.prepare_xml_input.get_processed_resources import _get_file_metadata
from dsp_tools.commands.xmlupload.prepare_xml_input.get_processed_resources import (
    _get_file_metadata_for_test_environments,
)
from dsp_tools.commands.xmlupload.prepare_xml_input.get_processed_resources import _get_one_processed_value
from dsp_tools.commands.xmlupload.prepare_xml_input.get_processed_resources import _get_one_resource
from dsp_tools.commands.xmlupload.prepare_xml_input.get_processed_resources import _resolve_file_value
from dsp_tools.commands.xmlupload.prepare_xml_input.get_processed_resources import _resolve_permission
from dsp_tools.commands.xmlupload.prepare_xml_input.get_processed_resources import get_processed_resources
from dsp_tools.error.exceptions import XmlUploadAuthorshipsNotFoundError
from dsp_tools.error.exceptions import XmlUploadListNodeNotFoundError
from dsp_tools.error.exceptions import XmlUploadPermissionsNotFoundError
from dsp_tools.legacy_models.datetimestamp import DateTimeStamp
from dsp_tools.utils.data_formats.date_util import Date
from dsp_tools.utils.rdflib_constants import KNORA_API_STR
from dsp_tools.utils.xml_parsing.models.parsed_resource import KnoraValueType
from dsp_tools.utils.xml_parsing.models.parsed_resource import ParsedFileValue
from dsp_tools.utils.xml_parsing.models.parsed_resource import ParsedFileValueMetadata
from dsp_tools.utils.xml_parsing.models.parsed_resource import ParsedMigrationMetadata
from dsp_tools.utils.xml_parsing.models.parsed_resource import ParsedResource
from dsp_tools.utils.xml_parsing.models.parsed_resource import ParsedValue

ONTO = "http://0.0.0.0:3333/ontology/9999/onto/v2#"
HAS_PROP = f"{ONTO}hasProp"
RES_TYPE = f"{ONTO}ResourceType"

IS_ON_PROD_LIKE_SERVER = True


@pytest.fixture
def lookups() -> XmlReferenceLookups:
    return XmlReferenceLookups(
        permissions={"public": Permissions({PermissionValue.CR: ["knora-admin:ProjectAdmin"]})},
        listnodes={
            ("list", "node"): "http://rdfh.ch/9999/node",
            ("", "http://rdfh.ch/9999/node"): "http://rdfh.ch/9999/node",
        },
        authorships={"auth_id": ["author"], "auth_id2": ["author1", "author2"]},
    )


@pytest.fixture
def file_with_permission() -> ParsedFileValue:
    metadata = ParsedFileValueMetadata("http://rdfh.ch/licenses/cc-by-nc-4.0", "copy", "auth_id", "public")
    return ParsedFileValue("file.jpg", KnoraValueType.STILL_IMAGE_FILE, metadata)


@pytest.fixture
def bool_value() -> ParsedValue:
    return ParsedValue(HAS_PROP, "true", KnoraValueType.BOOLEAN_VALUE, None, None)


@pytest.fixture
def iiif_file_value():
    metadata = ParsedFileValueMetadata("http://rdfh.ch/licenses/cc-by-nc-4.0", "copy", "auth_id", None)
    return ParsedFileValue("https://this/is/a/uri.jpg", KnoraValueType.STILL_IMAGE_IIIF, metadata)


class TestResources:
    def test_success(self, lookups):
        res = ParsedResource(
            res_id="id",
            res_type=RES_TYPE,
            label="lbl",
            permissions_id=None,
            values=[],
            file_value=None,
            migration_metadata=None,
        )
        result = get_processed_resources([res], lookups, IS_ON_PROD_LIKE_SERVER)
        assert len(result) == 1

    def test_failure(self, lookups: XmlReferenceLookups):
        res = ParsedResource(
            res_id="id",
            res_type=RES_TYPE,
            label="lbl",
            permissions_id="nonExisting",
            values=[],
            file_value=None,
            migration_metadata=None,
        )
        with pytest.raises(XmlUploadPermissionsNotFoundError):
            get_processed_resources([res], lookups, IS_ON_PROD_LIKE_SERVER)


class TestOneResource:
    def test_resource_one_value(self, bool_value, lookups: XmlReferenceLookups):
        res = ParsedResource(
            res_id="id",
            res_type=RES_TYPE,
            label="lbl",
            permissions_id=None,
            values=[bool_value],
            file_value=None,
            migration_metadata=None,
        )
        result = _get_one_resource(res, lookups, IS_ON_PROD_LIKE_SERVER)
        assert result.res_id == "id"
        assert result.type_iri == RES_TYPE
        assert result.label == "lbl"
        assert not result.permissions
        assert len(result.values) == 1
        assert not result.file_value
        assert not result.migration_metadata

    def test_resource_with_permissions(self, lookups: XmlReferenceLookups):
        res = ParsedResource(
            res_id="id",
            res_type=RES_TYPE,
            label="lbl",
            permissions_id="public",
            values=[],
            file_value=None,
            migration_metadata=None,
        )
        result = _get_one_resource(res, lookups, IS_ON_PROD_LIKE_SERVER)
        assert result.res_id == "id"
        assert result.type_iri == RES_TYPE
        assert result.label == "lbl"
        assert isinstance(result.permissions, Permissions)
        assert len(result.values) == 0
        assert not result.file_value
        assert not result.migration_metadata

    def test_with_ark(self, lookups: XmlReferenceLookups):
        parsed_metadata = ParsedMigrationMetadata(
            iri=None, ark="ark:/72163/4123-43xc6ivb931-a.2022829", creation_date="1999-12-31T23:59:59.9999999+01:00"
        )
        res = ParsedResource(
            res_id="id",
            res_type=RES_TYPE,
            label="lbl",
            permissions_id=None,
            values=[],
            file_value=None,
            migration_metadata=parsed_metadata,
        )
        result = _get_one_resource(res, lookups, IS_ON_PROD_LIKE_SERVER)
        assert result.res_id == "id"
        assert result.type_iri == RES_TYPE
        assert result.label == "lbl"
        assert not result.permissions
        assert len(result.values) == 0
        assert not result.file_value
        metadata = result.migration_metadata
        assert isinstance(metadata, MigrationMetadata)
        assert metadata.iri_str == "http://rdfh.ch/4123/5d5d1FKaUC2Wfl4zicggfg"
        assert metadata.creation_date == "1999-12-31T23:59:59.9999999+01:00"

    def test_with_iri(self, lookups: XmlReferenceLookups):
        parsed_metadata = ParsedMigrationMetadata(
            iri="http://rdfh.ch/4123/DiAmYQzQSzC7cdTo6OJMYA", ark=None, creation_date=None
        )
        res = ParsedResource(
            res_id="id",
            res_type=RES_TYPE,
            label="lbl",
            permissions_id=None,
            values=[],
            file_value=None,
            migration_metadata=parsed_metadata,
        )
        result = _get_one_resource(res, lookups, IS_ON_PROD_LIKE_SERVER)
        assert result.res_id == "id"
        assert result.type_iri == RES_TYPE
        assert result.label == "lbl"
        assert not result.permissions
        assert len(result.values) == 0
        assert not result.file_value
        metadata = result.migration_metadata
        assert isinstance(metadata, MigrationMetadata)
        assert metadata.iri_str == "http://rdfh.ch/4123/DiAmYQzQSzC7cdTo6OJMYA"
        assert not metadata.creation_date

    def test_resource_with_ark_and_iri(self, lookups: XmlReferenceLookups):
        parsed_metadata = ParsedMigrationMetadata(
            iri="http://rdfh.ch/4123/DiAmYQzQSzC7cdTo6OJMYA",
            ark="ark:/72163/4123-43xc6ivb931-a.2022829",
            creation_date="1999-12-31T23:59:59.9999999+01:00",
        )
        res = ParsedResource(
            res_id="id",
            res_type=RES_TYPE,
            label="lbl",
            permissions_id=None,
            values=[],
            file_value=None,
            migration_metadata=parsed_metadata,
        )
        result = _get_one_resource(res, lookups, IS_ON_PROD_LIKE_SERVER)
        assert result.res_id == "id"
        assert result.type_iri == RES_TYPE
        assert result.label == "lbl"
        assert not result.permissions
        assert len(result.values) == 0
        assert not result.file_value
        metadata = result.migration_metadata
        assert isinstance(metadata, MigrationMetadata)
        assert metadata.iri_str == "http://rdfh.ch/4123/5d5d1FKaUC2Wfl4zicggfg"
        time_stamp = metadata.creation_date
        assert isinstance(time_stamp, DateTimeStamp)
        assert time_stamp == DateTimeStamp("1999-12-31T23:59:59.9999999+01:00")

    def test_unknown_permission(self, lookups: XmlReferenceLookups):
        msg = regex.escape(r"Could not find permissions for value: unknown")
        res = ParsedResource(
            res_id="id",
            res_type=RES_TYPE,
            label="lbl",
            permissions_id="unknown",
            values=[],
            file_value=None,
            migration_metadata=None,
        )
        with pytest.raises(XmlUploadPermissionsNotFoundError, match=msg):
            _get_one_resource(res, lookups, IS_ON_PROD_LIKE_SERVER)

    def test_with_file_value(self, file_with_permission, lookups: XmlReferenceLookups):
        res = ParsedResource(
            res_id="id",
            res_type=RES_TYPE,
            label="lbl",
            permissions_id=None,
            values=[],
            file_value=file_with_permission,
            migration_metadata=None,
        )
        result = _get_one_resource(res, lookups, IS_ON_PROD_LIKE_SERVER)
        assert result.res_id == "id"
        assert result.type_iri == RES_TYPE
        assert result.label == "lbl"
        assert not result.permissions
        assert len(result.values) == 0
        file_val = result.file_value
        assert isinstance(file_val, ProcessedFileValue)
        assert file_val.value == "file.jpg"
        assert file_val.metadata.permissions
        assert not result.iiif_uri
        assert not result.migration_metadata

    def test_with_file_value_not_on_prod_missing_legal_info(self, lookups: XmlReferenceLookups):
        metadata = ParsedFileValueMetadata(None, None, None, None)
        parsed_file = ParsedFileValue("file.jpg", KnoraValueType.STILL_IMAGE_FILE, metadata)
        res = ParsedResource(
            res_id="id",
            res_type=RES_TYPE,
            label="lbl",
            permissions_id=None,
            values=[],
            file_value=parsed_file,
            migration_metadata=None,
        )
        result = _get_one_resource(res, lookups, is_on_prod_like_server=False)
        assert result.res_id == "id"
        assert result.type_iri == RES_TYPE
        assert result.label == "lbl"
        assert not result.permissions
        assert len(result.values) == 0
        file_val = result.file_value
        assert isinstance(file_val, ProcessedFileValue)
        assert file_val.value == "file.jpg"
        assert not file_val.metadata.permissions
        assert file_val.metadata.copyright_holder == "DUMMY"
        assert file_val.metadata.authorships == ["DUMMY"]
        assert file_val.metadata.license_iri == "http://rdfh.ch/licenses/unknown"
        assert not result.iiif_uri
        assert not result.migration_metadata

    def test_with_iiif_uri(self, iiif_file_value, lookups: XmlReferenceLookups):
        res = ParsedResource(
            res_id="id",
            res_type=RES_TYPE,
            label="lbl",
            permissions_id=None,
            values=[],
            file_value=iiif_file_value,
            migration_metadata=None,
        )
        result = _get_one_resource(res, lookups, IS_ON_PROD_LIKE_SERVER)
        assert result.res_id == "id"
        assert result.type_iri == RES_TYPE
        assert result.label == "lbl"
        assert not result.permissions
        assert len(result.values) == 0
        assert not result.file_value
        assert isinstance(result.iiif_uri, ProcessedIIIFUri)
        assert not result.migration_metadata


class TestFileValue:
    def test_resolve_file_value_iiif(self, iiif_file_value, lookups):
        resource = ParsedResource(
            res_id="id",
            res_type=RES_TYPE,
            label="lbl",
            permissions_id=None,
            values=[],
            file_value=iiif_file_value,
            migration_metadata=None,
        )
        file_res, iiif_res = _resolve_file_value(resource, lookups, IS_ON_PROD_LIKE_SERVER)
        assert file_res is None
        assert isinstance(iiif_res, ProcessedIIIFUri)
        result_metadata = iiif_res.metadata
        assert not result_metadata.permissions
        assert iiif_res.value == iiif_file_value.value
        assert result_metadata.license_iri == "http://rdfh.ch/licenses/cc-by-nc-4.0"
        assert result_metadata.copyright_holder == "copy"
        assert result_metadata.authorships == ["author"]

    def test_resolve_file_value_none(self, lookups):
        resource = ParsedResource(
            res_id="id",
            res_type=RES_TYPE,
            label="lbl",
            permissions_id=None,
            values=[],
            file_value=None,
            migration_metadata=None,
        )
        file_res, iiif_res = _resolve_file_value(resource, lookups, IS_ON_PROD_LIKE_SERVER)
        assert file_res is None
        assert iiif_res is None

    def test_resolve_file_value_file(self, file_with_permission, lookups):
        resource = ParsedResource(
            res_id="id",
            res_type=RES_TYPE,
            label="lbl",
            permissions_id=None,
            values=[],
            file_value=file_with_permission,
            migration_metadata=None,
        )
        file_res, iiif_res = _resolve_file_value(resource, lookups, IS_ON_PROD_LIKE_SERVER)
        assert iiif_res is None
        assert isinstance(file_res, ProcessedFileValue)
        result_metadata = file_res.metadata
        assert file_res.value == file_res.value
        assert file_res.file_type == file_with_permission.value_type
        assert isinstance(result_metadata.permissions, Permissions)
        assert result_metadata.license_iri == "http://rdfh.ch/licenses/cc-by-nc-4.0"
        assert result_metadata.copyright_holder == "copy"
        assert result_metadata.authorships == ["author"]


class TestFileMetadata:
    def test_prod_good_with_permissions(self, lookups):
        metadata = ParsedFileValueMetadata("http://rdfh.ch/licenses/cc-by-nc-4.0", "copy", "auth_id", "public")
        result_metadata = _get_file_metadata(metadata, lookups)
        assert isinstance(result_metadata.permissions, Permissions)
        assert result_metadata.license_iri == "http://rdfh.ch/licenses/cc-by-nc-4.0"
        assert result_metadata.copyright_holder == "copy"
        assert result_metadata.authorships == ["author"]

    def test_prod_good_without_permissions(self, lookups):
        metadata = ParsedFileValueMetadata("http://rdfh.ch/licenses/cc-by-nc-4.0", "copy", "auth_id2", None)
        result_metadata = _get_file_metadata(metadata, lookups)
        assert not result_metadata.permissions
        assert result_metadata.license_iri == "http://rdfh.ch/licenses/cc-by-nc-4.0"
        assert result_metadata.copyright_holder == "copy"
        assert result_metadata.authorships
        assert set(result_metadata.authorships) == {"author1", "author2"}

    def test_prod_with_unknown_authorship_id(self, lookups: XmlReferenceLookups):
        metadata = ParsedFileValueMetadata("http://rdfh.ch/licenses/cc-by-nc-4.0", "copy", "unkown_auth_id", None)
        with pytest.raises(XmlUploadAuthorshipsNotFoundError):
            _get_file_metadata(metadata, lookups)

    def test_get_file_metadata_for_test_environments_no_values(self, lookups):
        metadata = ParsedFileValueMetadata(None, None, None, None)
        result_metadata = _get_file_metadata_for_test_environments(metadata, lookups)
        assert not result_metadata.permissions
        assert result_metadata.license_iri == "http://rdfh.ch/licenses/unknown"
        assert result_metadata.copyright_holder == "DUMMY"
        assert result_metadata.authorships == ["DUMMY"]

    def test_get_file_metadata_for_test_environments_some_values(self, lookups):
        metadata = ParsedFileValueMetadata(None, "copy", None, None)
        result_metadata = _get_file_metadata_for_test_environments(metadata, lookups)
        assert not result_metadata.permissions
        assert result_metadata.license_iri == "http://rdfh.ch/licenses/unknown"
        assert result_metadata.copyright_holder == "copy"
        assert result_metadata.authorships == ["DUMMY"]

    def test_get_file_metadata_for_test_environments_all_values(self, lookups):
        metadata = ParsedFileValueMetadata("http://rdfh.ch/licenses/cc-by-nc-4.0", "copy", "auth_id", "public")
        result_metadata = _get_file_metadata_for_test_environments(metadata, lookups)
        assert isinstance(result_metadata.permissions, Permissions)
        assert result_metadata.license_iri == "http://rdfh.ch/licenses/cc-by-nc-4.0"
        assert result_metadata.copyright_holder == "copy"
        assert result_metadata.authorships == ["author"]


class TestValues:
    def test_bool_value(self, bool_value, lookups: XmlReferenceLookups):
        result = _get_one_processed_value(bool_value, lookups)
        assert isinstance(result, ProcessedBoolean)
        assert result.value == True  # noqa:E712 (Avoid equality comparisons)
        assert result.prop_iri == HAS_PROP
        assert not result.permissions
        assert not result.comment

    def test_bool_value_with_comment(self, lookups: XmlReferenceLookups):
        val = ParsedValue(HAS_PROP, "false", KnoraValueType.BOOLEAN_VALUE, None, "comment")
        result = _get_one_processed_value(val, lookups)
        assert result.value == False  # noqa:E712 (Avoid equality comparisons)
        assert result.prop_iri == HAS_PROP
        assert not result.permissions
        assert result.comment == "comment"

    def test_bool_value_with_permissions(self, lookups: XmlReferenceLookups):
        val = ParsedValue(HAS_PROP, "true", KnoraValueType.BOOLEAN_VALUE, "public", None)
        result = _get_one_processed_value(val, lookups)
        assert result.value == True  # noqa:E712 (Avoid equality comparisons)
        assert result.prop_iri == HAS_PROP
        assert isinstance(result.permissions, Permissions)
        assert not result.comment

    def test_bool_value_with_non_existing_permissions(self, lookups: XmlReferenceLookups):
        val = ParsedValue(HAS_PROP, "true", KnoraValueType.BOOLEAN_VALUE, "nonExisting", None)
        with pytest.raises(XmlUploadPermissionsNotFoundError):
            _get_one_processed_value(val, lookups)

    def test_color_value(self, lookups: XmlReferenceLookups):
        val = ParsedValue(HAS_PROP, "#5d1f1e", KnoraValueType.COLOR_VALUE, None, None)
        result = _get_one_processed_value(val, lookups)
        assert isinstance(result, ProcessedColor)
        assert result.value == "#5d1f1e"
        assert result.prop_iri == HAS_PROP
        assert not result.permissions
        assert not result.comment

    def test_date_value(self, lookups: XmlReferenceLookups):
        val = ParsedValue(HAS_PROP, "CE:1849:CE:1850", KnoraValueType.DATE_VALUE, None, None)
        result = _get_one_processed_value(val, lookups)
        assert isinstance(result, ProcessedDate)
        assert isinstance(result.value, Date)
        assert result.prop_iri == HAS_PROP
        assert not result.permissions
        assert not result.comment

    def test_decimal_value(self, lookups: XmlReferenceLookups):
        val = ParsedValue(HAS_PROP, "1.4", KnoraValueType.DECIMAL_VALUE, None, None)
        result = _get_one_processed_value(val, lookups)
        assert isinstance(result, ProcessedDecimal)
        assert result.value == 1.4
        assert result.prop_iri == HAS_PROP
        assert not result.permissions
        assert not result.comment

    def test_geometry_value(self, lookups: XmlReferenceLookups):
        val = ParsedValue(f"{KNORA_API_STR}hasGeometry", "{}", KnoraValueType.GEOM_VALUE, None, None)
        result = _get_one_processed_value(val, lookups)
        assert isinstance(result, ProcessedGeometry)
        assert result.value == "{}"
        assert result.prop_iri == f"{KNORA_API_STR}hasGeometry"
        assert not result.permissions
        assert not result.comment

    def test_geoname_value(self, lookups: XmlReferenceLookups):
        val = ParsedValue(HAS_PROP, "5416656", KnoraValueType.GEONAME_VALUE, None, None)
        result = _get_one_processed_value(val, lookups)
        assert isinstance(result, ProcessedGeoname)
        assert result.value == "5416656"
        assert result.prop_iri == HAS_PROP
        assert not result.permissions
        assert not result.comment

    def test_integer_value(self, lookups: XmlReferenceLookups):
        val = ParsedValue(HAS_PROP, "1", KnoraValueType.INT_VALUE, None, None)
        result = _get_one_processed_value(val, lookups)
        assert isinstance(result, ProcessedInt)
        assert result.value == 1
        assert result.prop_iri == HAS_PROP
        assert not result.permissions
        assert not result.comment

    def test_interval_value(self, lookups: XmlReferenceLookups):
        val = ParsedValue(f"{KNORA_API_STR}hasSegmentBounds", ("1", "2"), KnoraValueType.INTERVAL_VALUE, None, None)
        result = _get_one_processed_value(val, lookups)
        assert isinstance(result, ProcessedInterval)
        assert result.value.start == 1.0
        assert result.value.end == 2.0
        assert result.prop_iri == f"{KNORA_API_STR}hasSegmentBounds"
        assert not result.permissions
        assert not result.comment

    def test_list_value(self, lookups: XmlReferenceLookups):
        val = ParsedValue(HAS_PROP, ("list", "node"), KnoraValueType.LIST_VALUE, "public", "cmt")
        result = _get_one_processed_value(val, lookups)
        assert isinstance(result, ProcessedList)
        assert result.value == "http://rdfh.ch/9999/node"
        assert result.prop_iri == HAS_PROP
        assert isinstance(result.permissions, Permissions)
        assert result.comment == "cmt"

    def test_list_value_raises(self, lookups: XmlReferenceLookups):
        val = ParsedValue(HAS_PROP, ("unknown", "unknown"), KnoraValueType.LIST_VALUE, None, None)
        with pytest.raises(XmlUploadListNodeNotFoundError):
            _get_one_processed_value(val, lookups)

    def test_list_value_with_iri(self, lookups: XmlReferenceLookups):
        val = ParsedValue(HAS_PROP, ("", "http://rdfh.ch/9999/node"), KnoraValueType.LIST_VALUE, "public", "cmt")
        result = _get_one_processed_value(val, lookups)
        assert isinstance(result, ProcessedList)
        assert result.value == "http://rdfh.ch/9999/node"
        assert result.prop_iri == HAS_PROP
        assert isinstance(result.permissions, Permissions)
        assert result.comment == "cmt"

    def test_simple_text_value(self, lookups: XmlReferenceLookups):
        val = ParsedValue(HAS_PROP, "text", KnoraValueType.SIMPLETEXT_VALUE, None, None)
        result = _get_one_processed_value(val, lookups)
        assert isinstance(result, ProcessedSimpleText)
        assert result.value == "text"
        assert result.prop_iri == HAS_PROP
        assert not result.permissions
        assert not result.comment

    def test_richtext_value(self, lookups: XmlReferenceLookups):
        text_str = "<text>this is text</text>"
        val = ParsedValue(HAS_PROP, text_str, KnoraValueType.RICHTEXT_VALUE, "public", "cmt")
        result = _get_one_processed_value(val, lookups)
        assert isinstance(result, ProcessedRichtext)
        assert result.value.xmlstr == text_str
        assert result.prop_iri == HAS_PROP
        assert isinstance(result.permissions, Permissions)
        assert result.comment == "cmt"
        assert result.resource_references == set()

    def test_richtext_value_with_standoff(self, lookups: XmlReferenceLookups):
        text_str = 'Comment with <a class="salsah-link" href="IRI:link:IRI">link text</a>.'
        val = ParsedValue(HAS_PROP, text_str, KnoraValueType.RICHTEXT_VALUE, "public", "cmt")
        result = _get_one_processed_value(val, lookups)
        assert isinstance(result, ProcessedRichtext)
        assert result.value.xmlstr == text_str
        assert result.prop_iri == HAS_PROP
        assert isinstance(result.permissions, Permissions)
        assert result.comment == "cmt"
        assert result.resource_references == {"link"}

    def test_link_value(self, lookups: XmlReferenceLookups):
        val = ParsedValue(HAS_PROP, "other_id", KnoraValueType.LINK_VALUE, "public", "cmt")
        result = _get_one_processed_value(val, lookups)
        assert isinstance(result, ProcessedLink)
        assert result.value == "other_id"
        assert result.prop_iri == f"{HAS_PROP}Value"
        assert isinstance(result.permissions, Permissions)
        assert result.comment == "cmt"

    def test_time_value(self, lookups: XmlReferenceLookups):
        val = ParsedValue(HAS_PROP, "2019-10-23T13:45:12.01-14:00", KnoraValueType.TIME_VALUE, None, None)
        result = _get_one_processed_value(val, lookups)
        assert isinstance(result, ProcessedTime)
        assert result.value == "2019-10-23T13:45:12.01-14:00"
        assert result.prop_iri == HAS_PROP
        assert not result.permissions
        assert not result.comment

    def test_uri_value(self, lookups: XmlReferenceLookups):
        val = ParsedValue(HAS_PROP, "https://dasch.swiss", KnoraValueType.URI_VALUE, None, None)
        result = _get_one_processed_value(val, lookups)
        assert isinstance(result, ProcessedUri)
        assert result.value == "https://dasch.swiss"
        assert result.prop_iri == HAS_PROP
        assert not result.permissions
        assert not result.comment


class TestPermissions:
    def test_good(self, lookups):
        result = _resolve_permission("public", lookups.permissions)
        assert isinstance(result, Permissions)
        assert str(result) == "CR knora-admin:ProjectAdmin"

    def test_none(self, lookups):
        result = _resolve_permission(None, lookups.permissions)
        assert not result

    def test_empty_string(self, lookups):
        result = _resolve_permission("", lookups.permissions)
        assert not result

    def test_raises(self, lookups):
        msg = regex.escape("Could not find permissions for value: inexistent")
        with pytest.raises(XmlUploadPermissionsNotFoundError, match=msg):
            _resolve_permission("inexistent", lookups.permissions)



from typing import Any

import pytest
import regex
from lxml import etree

from dsp_tools.commands.id2iri import _remove_resources_if_id_in_mapping
from dsp_tools.commands.id2iri import _replace_ids_by_iris
from dsp_tools.commands.id2iri import id2iri
from dsp_tools.error.custom_warnings import DspToolsUserWarning
from dsp_tools.error.exceptions import BaseError


@pytest.fixture
def mapping() -> dict[str, str]:
    return {
        "test_thing_0": "http://rdfh.ch/082E/-lRvrg7tQI6aVpcTJbVrwg",
        "test_thing_1": "http://rdfh.ch/082E/JK63OpYWTDWNYVOYFN7FdQ",
        "test_thing_2": "http://rdfh.ch/082E/1l63Oasdfopiujlkmn78ak",
        "test_thing_with_ark_1": "http://rdfh.ch/082E/qwasddoiu8_6flkjh67dss",
    }


def test_invalid_xml_file_name() -> None:
    with pytest.raises(BaseError, match=regex.escape("File test.xml could not be found")):
        id2iri(
            xml_file="test.xml",
            json_file="testdata/id2iri/test-id2iri-mapping.json",
        )


def test_invalid_json_file_name() -> None:
    with pytest.raises(BaseError, match=regex.escape("File test.json could not be found")):
        id2iri(
            xml_file="testdata/id2iri/test-id2iri-data.xml",
            json_file="test.json",
        )


def test_replace_ids_by_iris_no_replacements(mapping: dict[str, Any]) -> None:
    xml = """
    <knora shortcode="4123" default-ontology="testonto">
        <resource label="no_replacements" restype=":TestThing" id="no_replacements">
            <text-prop name=":hasRichtext">
                <text encoding="xml"><a class="salsa-link" href="IRI:id_that_is_not_in_mapping:IRI">link</a></text>
            </text-prop>
        </resource>
    </knora>
    """
    xml = regex.sub(r"^(\n +)|(\n +)$", "", xml)
    root_replaced = _replace_ids_by_iris(etree.fromstring(xml), mapping)
    xml_replaced = etree.tostring(root_replaced).decode("utf-8")
    assert xml_replaced == xml


def test_replace_ids_by_iris_resptr_only(mapping: dict[str, Any]) -> None:
    xml = """
    <knora shortcode="4123" default-ontology="testonto">
        <resource label="resptr_only" restype=":TestThing" id="resptr_only">
            <text-prop name=":hasRichtext">
                <text encoding="xml">Text</text>
            </text-prop>
            <resptr-prop name=":hasTestThing2">
                <resptr>test_thing_0</resptr>
                <resptr>test_thing_with_ark_1</resptr>
            </resptr-prop>
        </resource>
    </knora>
    """
    xml_expected = """
    <knora shortcode="4123" default-ontology="testonto">
        <resource label="resptr_only" restype=":TestThing" id="resptr_only">
            <text-prop name=":hasRichtext">
                <text encoding="xml">Text</text>
            </text-prop>
            <resptr-prop name=":hasTestThing2">
                <resptr>http://rdfh.ch/082E/-lRvrg7tQI6aVpcTJbVrwg</resptr>
                <resptr>http://rdfh.ch/082E/qwasddoiu8_6flkjh67dss</resptr>
            </resptr-prop>
        </resource>
    </knora>
    """
    xml_expected = regex.sub(r"^(\n +)|(\n +)$", "", xml_expected)
    root_replaced = _replace_ids_by_iris(etree.fromstring(xml), mapping)
    xml_replaced = etree.tostring(root_replaced).decode("utf-8")
    assert xml_replaced == xml_expected


def test_replace_ids_by_iris_salsah_link_only(mapping: dict[str, Any]) -> None:
    xml = """
    <knora shortcode="4123" default-ontology="testonto">
        <resource label="salsah_link_only" restype=":TestThing" id="salsah_link_only">
            <text-prop name=":hasRichtext">
                <text encoding="xml">
                    Text with a <a class="salsah-link" href="IRI:test_thing_1:IRI">
                    link to test_thing_1</a> and <strong>a bold <em>and italicized
                    <a class="salsah-link" href="IRI:test_thing_2:IRI">
                    link to test_thing_2</a></em></strong>and a 
                    <a class="salsah-link" href="IRI:test_thing_with_ark_1:IRI">link</a> 
                </text>
            </text-prop>
        </resource>
    </knora>
    """
    xml_expected = """
    <knora shortcode="4123" default-ontology="testonto">
        <resource label="salsah_link_only" restype=":TestThing" id="salsah_link_only">
            <text-prop name=":hasRichtext">
                <text encoding="xml">
                    Text with a <a class="salsah-link" href="http://rdfh.ch/082E/JK63OpYWTDWNYVOYFN7FdQ">
                    link to test_thing_1</a> and <strong>a bold <em>and italicized
                    <a class="salsah-link" href="http://rdfh.ch/082E/1l63Oasdfopiujlkmn78ak">
                    link to test_thing_2</a></em></strong>and a 
                    <a class="salsah-link" href="http://rdfh.ch/082E/qwasddoiu8_6flkjh67dss">link</a> 
                </text>
            </text-prop>
        </resource>
    </knora>
    """
    xml_expected = regex.sub(r"^(\n +)|(\n +)$", "", xml_expected)
    root_replaced = _replace_ids_by_iris(etree.fromstring(xml), mapping)
    xml_replaced = etree.tostring(root_replaced).decode("utf-8")
    assert xml_replaced == xml_expected


def test_replace_ids_by_iris_resptr_and_salsah_link(mapping: dict[str, Any]) -> None:
    xml = """
    <knora shortcode="4123" default-ontology="testonto">
        <resource label="resptr_and_salsah_link" restype=":TestThing" id="resptr_and_salsah_link">
            <text-prop name=":hasRichtext">
                <text encoding="xml">
                    Text with a <a class="salsah-link" href="IRI:test_thing_1:IRI">
                    link to test_thing_1</a> and <strong>a bold <em>and italicized
                    <a class="salsah-link" href="IRI:test_thing_2:IRI">
                    link to test_thing_2</a></em></strong>and a 
                    <a class="salsah-link" href="IRI:test_thing_with_ark_1:IRI">link</a> 
                </text>
            </text-prop>
            <resptr-prop name=":hasTestThing2">
                <resptr>test_thing_0</resptr>
                <resptr>test_thing_with_ark_1</resptr>
            </resptr-prop>
        </resource>
    </knora>
    """
    xml_expected = """
    <knora shortcode="4123" default-ontology="testonto">
        <resource label="resptr_and_salsah_link" restype=":TestThing" id="resptr_and_salsah_link">
            <text-prop name=":hasRichtext">
                <text encoding="xml">
                    Text with a <a class="salsah-link" href="http://rdfh.ch/082E/JK63OpYWTDWNYVOYFN7FdQ">
                    link to test_thing_1</a> and <strong>a bold <em>and italicized
                    <a class="salsah-link" href="http://rdfh.ch/082E/1l63Oasdfopiujlkmn78ak">
                    link to test_thing_2</a></em></strong>and a 
                    <a class="salsah-link" href="http://rdfh.ch/082E/qwasddoiu8_6flkjh67dss">link</a> 
                </text>
            </text-prop>
            <resptr-prop name=":hasTestThing2">
                <resptr>http://rdfh.ch/082E/-lRvrg7tQI6aVpcTJbVrwg</resptr>
                <resptr>http://rdfh.ch/082E/qwasddoiu8_6flkjh67dss</resptr>
            </resptr-prop>
        </resource>
    </knora>
    """
    xml_expected = regex.sub(r"^(\n +)|(\n +)$", "", xml_expected)
    root_replaced = _replace_ids_by_iris(etree.fromstring(xml), mapping)
    xml_replaced = etree.tostring(root_replaced).decode("utf-8")
    assert xml_replaced == xml_expected


def test_replace_ids_by_iris_segment(mapping: dict[str, Any]) -> None:
    xml = """
    <knora shortcode="4123" default-ontology="testonto">
        <audio-segment label="Audio Segment with all possible values" id="audio_segment_0">
            <isSegmentOf>audio_thing_1</isSegmentOf>
            <hasSegmentBounds segment_start="3.5" segment_end="5.6"/>
            <hasTitle>Title of audio segment</hasTitle>
            <hasComment>
                <a class="salsah-link" href="IRI:test_thing_1:IRI">link to test_thing_1</a>
            </hasComment>
            <hasDescription>
                <a class="salsah-link" href="IRI:test_thing_1:IRI">link to test_thing_1</a>
            </hasDescription>
            <hasKeyword>Keyword 2 of audio segment</hasKeyword>
            <relatesTo>test_thing_1</relatesTo>
        </audio-segment>
    </knora>
    """
    xml_expected = """
    <knora shortcode="4123" default-ontology="testonto">
        <audio-segment label="Audio Segment with all possible values" id="audio_segment_0">
            <isSegmentOf>audio_thing_1</isSegmentOf>
            <hasSegmentBounds segment_start="3.5" segment_end="5.6"/>
            <hasTitle>Title of audio segment</hasTitle>
            <hasComment>
                <a class="salsah-link" href="http://rdfh.ch/082E/JK63OpYWTDWNYVOYFN7FdQ">link to test_thing_1</a>
            </hasComment>
            <hasDescription>
                <a class="salsah-link" href="http://rdfh.ch/082E/JK63OpYWTDWNYVOYFN7FdQ">link to test_thing_1</a>
            </hasDescription>
            <hasKeyword>Keyword 2 of audio segment</hasKeyword>
            <relatesTo>http://rdfh.ch/082E/JK63OpYWTDWNYVOYFN7FdQ</relatesTo>
        </audio-segment>
    </knora>
    """
    xml_expected = regex.sub(r"^(\n +)|(\n +)$", "", xml_expected)
    root_replaced = _replace_ids_by_iris(etree.fromstring(xml), mapping)
    xml_replaced = etree.tostring(root_replaced).decode("utf-8")
    assert xml_replaced == xml_expected


def test_id2iri_remove_resources(mapping: dict[str, Any]) -> None:
    xml = """
    <knora shortcode="4123" default-ontology="testonto">
        <resource label="test_thing_0" restype=":TestThing" id="test_thing_0">
            <text-prop name=":hasRichtext"><text encoding="xml">Hello world</text></text-prop>
        </resource>
        <resource label="id_that_is_not_in_mapping" restype=":TestThing" id="id_that_is_not_in_mapping">
            <resptr-prop name=":foo"><resptr>test_thing_0</resptr></resptr-prop>
        </resource>
        <resource label="test_thing_1" restype=":TestThing" id="test_thing_1">
            <resptr-prop name=":foo"><resptr>test_thing_0</resptr></resptr-prop>
        </resource>
    </knora>
    """
    xml_expected = """
    <knora shortcode="4123" default-ontology="testonto">
        <resource label="id_that_is_not_in_mapping" restype=":TestThing" id="id_that_is_not_in_mapping">
            <resptr-prop name=":foo"><resptr>test_thing_0</resptr></resptr-prop>
        </resource>
    </knora>
    """
    xml_expected = xml_expected.replace("\n", "").replace(" ", "")
    match = regex.escape(r"Removed 2/3 resources from the XML file, because their ID was in the mapping")
    with pytest.warns(DspToolsUserWarning, match=match):
        tree_returned = _remove_resources_if_id_in_mapping(
            tree=etree.fromstring(xml),
            mapping=mapping,
        )
    xml_returned = etree.tostring(tree_returned, pretty_print=False).decode("utf-8")
    xml_returned = xml_returned.replace("\n", "").replace(" ", "")
    assert xml_returned == xml_expected


if __name__ == "__main__":
    pytest.main([__file__])



# mypy: disable-error-code="no-untyped-def"

from pathlib import Path

import pytest

from dsp_tools.cli.args import ServerCredentials
from dsp_tools.cli.args import ValidateDataConfig
from dsp_tools.cli.args import ValidationSeverity
from dsp_tools.clients.authentication_client import AuthenticationClient
from dsp_tools.clients.authentication_client_live import AuthenticationClientLive
from dsp_tools.commands.validate_data.models.input_problems import ProblemType
from dsp_tools.commands.validate_data.models.input_problems import SortedProblems
from dsp_tools.commands.validate_data.models.input_problems import ValidateDataResult
from dsp_tools.commands.validate_data.prepare_data.prepare_data import get_info_and_parsed_resources_from_file
from dsp_tools.commands.validate_data.process_validation_report.get_user_validation_message import sort_user_problems
from dsp_tools.commands.validate_data.process_validation_report.query_validation_result import reformat_validation_graph
from dsp_tools.commands.validate_data.shacl_cli_validator import ShaclCliValidator
from dsp_tools.commands.validate_data.validate_data import _get_validation_status
from dsp_tools.commands.validate_data.validate_data import _validate_data
from dsp_tools.commands.validate_data.validate_data import validate_parsed_resources
from dsp_tools.commands.validate_data.validation.get_validation_report import get_validation_report
from test.e2e.commands.validate_data.util import prepare_data_for_validation_from_file

# ruff: noqa: ARG001 Unused function argument


CONFIG = ValidateDataConfig(
    xml_file=Path(),
    save_graph_dir=None,
    severity=ValidationSeverity.INFO,
    ignore_duplicate_files_warning=False,
    is_on_prod_server=False,
)


@pytest.fixture(scope="module")
def authentication(creds: ServerCredentials) -> AuthenticationClient:
    auth = AuthenticationClientLive(server=creds.server, email=creds.user, password=creds.password)
    return auth


@pytest.fixture(scope="module")
def no_violations_with_warnings(
    create_generic_project, authentication, shacl_validator: ShaclCliValidator
) -> ValidateDataResult:
    file = Path("testdata/validate-data/generic/no_violations_with_warnings.xml")
    graphs, used_iris = prepare_data_for_validation_from_file(
        file, authentication, CONFIG.ignore_duplicate_files_warning
    )
    return _validate_data(graphs, used_iris, CONFIG)


@pytest.fixture(scope="module")
def no_violations_with_info(
    create_generic_project, authentication, shacl_validator: ShaclCliValidator
) -> ValidateDataResult:
    file = Path("testdata/validate-data/generic/no_violations_with_info.xml")
    graphs, used_iris = prepare_data_for_validation_from_file(
        file, authentication, CONFIG.ignore_duplicate_files_warning
    )
    return _validate_data(graphs, used_iris, CONFIG)


@pytest.fixture(scope="module")
def parsed_resources_51_duplicate_images(create_generic_project, authentication):
    file = Path("testdata/validate-data/generic/51_duplicate_images.xml")
    parsed_resources, shortcode, authorship_lookup, permission_ids = get_info_and_parsed_resources_from_file(
        file, authentication.server
    )
    return parsed_resources, authorship_lookup, permission_ids, shortcode


class TestGetDuplicateFileValidationResult:
    def test_ignore_files_on_test_env(self, parsed_resources_51_duplicate_images, authentication):
        parsed_resources, authorship_lookup, permission_ids, shortcode = parsed_resources_51_duplicate_images
        config = ValidateDataConfig(
            xml_file=Path(),
            save_graph_dir=None,
            severity=ValidationSeverity.INFO,
            ignore_duplicate_files_warning=True,
            is_on_prod_server=False,
        )
        validation_passed = validate_parsed_resources(
            parsed_resources=parsed_resources,
            authorship_lookup=authorship_lookup,
            permission_ids=permission_ids,
            shortcode=shortcode,
            config=config,
            auth=authentication,
        )
        assert validation_passed

    def test_do_not_ignore_files_on_test_env(self, parsed_resources_51_duplicate_images, authentication):
        parsed_resources, authorship_lookup, permission_ids, shortcode = parsed_resources_51_duplicate_images
        config = ValidateDataConfig(
            xml_file=Path(),
            save_graph_dir=None,
            severity=ValidationSeverity.INFO,
            ignore_duplicate_files_warning=False,
            is_on_prod_server=False,
        )
        validation_passed = validate_parsed_resources(
            parsed_resources=parsed_resources,
            authorship_lookup=authorship_lookup,
            permission_ids=permission_ids,
            shortcode=shortcode,
            config=config,
            auth=authentication,
        )
        assert validation_passed

    def test_ignore_files_on_prod_env(self, parsed_resources_51_duplicate_images, authentication):
        parsed_resources, authorship_lookup, permission_ids, shortcode = parsed_resources_51_duplicate_images
        config = ValidateDataConfig(
            xml_file=Path(),
            save_graph_dir=None,
            severity=ValidationSeverity.INFO,
            ignore_duplicate_files_warning=True,
            is_on_prod_server=True,
        )
        validation_passed = validate_parsed_resources(
            parsed_resources=parsed_resources,
            authorship_lookup=authorship_lookup,
            permission_ids=permission_ids,
            shortcode=shortcode,
            config=config,
            auth=authentication,
        )
        assert validation_passed

    def test_do_not_ignore_files_on_prod_env(self, parsed_resources_51_duplicate_images, authentication):
        parsed_resources, authorship_lookup, permission_ids, shortcode = parsed_resources_51_duplicate_images
        config = ValidateDataConfig(
            xml_file=Path(),
            save_graph_dir=None,
            severity=ValidationSeverity.INFO,
            ignore_duplicate_files_warning=False,
            is_on_prod_server=True,
        )
        validation_passed = validate_parsed_resources(
            parsed_resources=parsed_resources,
            authorship_lookup=authorship_lookup,
            permission_ids=permission_ids,
            shortcode=shortcode,
            config=config,
            auth=authentication,
        )
        assert not validation_passed


class TestGetCorrectValidationResult:
    def test_no_violations_with_warnings_not_on_prod(self, no_violations_with_warnings):
        # this boolean carries the information if there are problems of any severity level,
        # but not if the validation will pass
        assert not no_violations_with_warnings.no_problems
        sorted_problems = no_violations_with_warnings.problems
        assert isinstance(sorted_problems, SortedProblems)
        result = _get_validation_status(sorted_problems, is_on_prod=False)
        assert result is True

    def test_no_violations_with_warnings_on_prod(self, no_violations_with_warnings):
        sorted_problems = no_violations_with_warnings.problems
        assert isinstance(sorted_problems, SortedProblems)
        result = _get_validation_status(sorted_problems, is_on_prod=True)
        assert result is False

    def test_no_violations_with_info_not_on_prod(self, no_violations_with_info):
        # this boolean carries the information if there are problems of any severity level,
        # but not if the validation will pass
        assert not no_violations_with_info.no_problems
        sorted_problems = no_violations_with_info.problems
        assert isinstance(sorted_problems, SortedProblems)
        result = _get_validation_status(sorted_problems, is_on_prod=False)
        assert result is True

    def test_no_violations_with_info_on_prod(self, no_violations_with_info):
        sorted_problems = no_violations_with_info.problems
        assert isinstance(sorted_problems, SortedProblems)
        result = _get_validation_status(sorted_problems, is_on_prod=True)
        assert result is True


class TestSortedProblems:
    def test_no_violations_with_warnings_problems(self, no_violations_with_warnings):
        expected_warnings = [
            # each type of missing legal info (authorship, copyright, license) produces one violation
            ("archive_no_legal_info", ProblemType.GENERIC),
            ("archive_no_legal_info", ProblemType.GENERIC),
            ("archive_no_legal_info", ProblemType.GENERIC),
            ("iiif_no_legal_info", ProblemType.GENERIC),
            ("iiif_no_legal_info", ProblemType.GENERIC),
            ("iiif_no_legal_info", ProblemType.GENERIC),
            ("image_no_legal_info", ProblemType.GENERIC),
            ("image_no_legal_info", ProblemType.GENERIC),
            ("image_no_legal_info", ProblemType.GENERIC),
        ]
        sorted_problems = no_violations_with_warnings.problems
        assert isinstance(sorted_problems, SortedProblems)
        sorted_warnings = sorted(sorted_problems.user_warnings, key=lambda x: str(x.res_id))
        assert not sorted_problems.unique_violations
        assert len(sorted_problems.user_warnings) == len(expected_warnings)
        assert not sorted_problems.user_info
        assert not sorted_problems.unexpected_shacl_validation_components
        for one_result, expected_info in zip(sorted_warnings, expected_warnings):
            assert one_result.problem_type == expected_info[1]
            assert one_result.res_id == expected_info[0]

    def test_no_violations_with_info(self, no_violations_with_info):
        all_expected_info = [
            ("duplicate_iiif_1", ProblemType.FILE_DUPLICATE),
            ("duplicate_iiif_2", ProblemType.FILE_DUPLICATE),
            ("duplicate_still_image_1", ProblemType.FILE_DUPLICATE),
            ("duplicate_still_image_2", ProblemType.FILE_DUPLICATE),
            ("link_to_resource_in_db", ProblemType.INEXISTENT_LINKED_RESOURCE),
            ("triplicate_archive_1", ProblemType.FILE_DUPLICATE),
            ("triplicate_archive_2", ProblemType.FILE_DUPLICATE),
            ("triplicate_archive_3", ProblemType.FILE_DUPLICATE),
        ]
        sorted_problems = no_violations_with_info.problems
        assert isinstance(sorted_problems, SortedProblems)
        sorted_info = sorted(sorted_problems.user_info, key=lambda x: str(x.res_id))
        assert not sorted_problems.unique_violations
        assert not sorted_problems.user_warnings
        assert len(sorted_problems.user_info) == len(all_expected_info)
        assert not sorted_problems.unexpected_shacl_validation_components
        for one_result, expected_info in zip(sorted_info, all_expected_info):
            assert one_result.problem_type == expected_info[1]
            assert one_result.res_id == expected_info[0]

    def test_no_violations_with_info_ignore_duplicate_files_warning(self, authentication, shacl_validator):
        file = Path("testdata/validate-data/generic/no_violations_with_info.xml")
        config = ValidateDataConfig(
            xml_file=Path(),
            save_graph_dir=None,
            severity=ValidationSeverity.INFO,
            ignore_duplicate_files_warning=True,
            is_on_prod_server=False,
        )
        graphs, used_iris = prepare_data_for_validation_from_file(
            file, authentication, config.ignore_duplicate_files_warning
        )
        report = get_validation_report(graphs, shacl_validator)
        reformatted = reformat_validation_graph(report)
        no_violations_with_info = sort_user_problems(reformatted)
        all_expected_info = [
            ("duplicate_iiif_1", ProblemType.FILE_DUPLICATE),
            ("duplicate_iiif_2", ProblemType.FILE_DUPLICATE),
            ("link_to_resource_in_db", ProblemType.INEXISTENT_LINKED_RESOURCE),
        ]
        sorted_info = sorted(no_violations_with_info.user_info, key=lambda x: str(x.res_id))
        assert not no_violations_with_info.unique_violations
        assert not no_violations_with_info.user_warnings
        assert len(no_violations_with_info.user_info) == len(all_expected_info)
        assert not no_violations_with_info.unexpected_shacl_validation_components
        for one_result, expected_info in zip(sorted_info, all_expected_info):
            assert one_result.problem_type == expected_info[1]
            assert one_result.res_id == expected_info[0]


if __name__ == "__main__":
    pytest.main([__file__])



from dataclasses import dataclass

from dsp_tools.clients.legal_info_client import LegalInfoClient
from dsp_tools.commands.xmlupload.models.ingest import AssetClient
from dsp_tools.commands.xmlupload.prepare_xml_input.list_client import ListClient
from dsp_tools.commands.xmlupload.project_client import ProjectClient


@dataclass(frozen=True)
class UploadClients:
    """Holds all the clients needed for the upload process."""

    asset_client: AssetClient
    project_client: ProjectClient
    list_client: ListClient
    legal_info_client: LegalInfoClient



from dataclasses import dataclass
from pathlib import Path

import pandas as pd
import regex

separator = "\n\n"
list_separator = "\n- "
list_separator_indented = "\n    - "


@dataclass(frozen=True)
class UploadFailure:
    """Information on why the upload of a file to the ingest server failed."""

    filepath: Path
    reason: str
    status_code: int | None = None
    response_text: str | None = None


@dataclass(frozen=True)
class UploadFailures:
    """Aggregated information of all failed uploads."""

    failures: list[UploadFailure]
    num_of_initial_files: int
    shortcode: str
    dsp_ingest_url: str
    maximum_prints: int = 50

    def execute_error_protocol(self) -> str:
        """
        Generate the error message to communicate the problems to the user.
        If there are too many problems, save them to a file.

        Returns:
            error message
        """
        ratio = f"{self.num_of_initial_files - len(self.failures)}/{self.num_of_initial_files}"
        msg = f"Uploaded {ratio} files onto server {self.dsp_ingest_url}. "
        if len(self.failures) > self.maximum_prints:
            url = regex.sub(r"https?://", "", self.dsp_ingest_url)
            output_file = Path(f"upload_failures_{self.shortcode}_{url}.csv")
            self._save_to_csv(output_file)
            msg += f"Failed to upload {len(self.failures)} files. "
            msg += f"The full list of failed files has been saved to '{output_file}'."
        else:
            msg += f"Failed to upload the following {len(self.failures)} files:"
            for f in self.failures:
                msg += list_separator + f"{f.filepath}: {f.reason}"
                if f.status_code:
                    msg += list_separator_indented + f"Status code: {f.status_code}"
                if f.response_text:
                    msg += list_separator_indented + f"Response text: {f.response_text}"
        return msg

    def _save_to_csv(self, output_file: Path) -> None:
        data = {
            "Filepath": [failure.filepath for failure in self.failures],
            "Reason": [failure.reason for failure in self.failures],
            "Status code": [failure.status_code for failure in self.failures],
            "Response text": [failure.response_text for failure in self.failures],
        }
        df = pd.DataFrame(data)
        df.to_csv(output_file, index=False)
